@@ -9,20 +9,28 @@
 
 #include <dlfcn.h>
 
+#include <algorithm>
+
 #include "base/check.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
-#include "base/cxx17_backports.h"
+#include "base/environment.h"
 #include "base/memory/raw_ptr.h"
+#include "base/nix/xdg_util.h"
 #include "base/notreached.h"
 #include "base/path_service.h"
+#include "base/scoped_environment_variable_override.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
+#include "cc/paint/paint_canvas.h"
 #include "chrome/browser/themes/theme_properties.h"  // nogncheck
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/ime/linux/linux_input_method_context.h"
 #include "ui/color/color_mixer.h"
 #include "ui/color/color_provider.h"
+#include "ui/color/color_provider_manager.h"
 #include "ui/color/color_recipe.h"
+#include "ui/color/color_transform.h"
 #include "ui/gfx/color_palette.h"
 #include "ui/gfx/color_utils.h"
 #include "ui/gfx/font.h"
@@ -31,19 +39,60 @@
 #include "ui/gfx/image/image.h"
 #include "ui/gfx/image/image_skia_rep.h"
 #include "ui/gfx/image/image_skia_source.h"
+#include "ui/linux/device_scale_factor_observer.h"
 #include "ui/linux/linux_ui.h"
+#include "ui/linux/linux_ui_delegate.h"
 #include "ui/linux/nav_button_provider.h"
 #include "ui/native_theme/native_theme_aura.h"
 #include "ui/native_theme/native_theme_base.h"
 #include "ui/qt/qt_interface.h"
+#include "ui/shell_dialogs/select_file_dialog.h"
 #include "ui/shell_dialogs/select_file_policy.h"
-#include "ui/shell_dialogs/shell_dialog_linux.h"
 #include "ui/views/controls/button/label_button_border.h"
 
 namespace qt {
 
 namespace {
 
+const char kQtVersionFlag[] = "qt-version";
+
+void* LoadLibrary(const base::FilePath& path) {
+  return dlopen(path.value().c_str(), RTLD_NOW | RTLD_GLOBAL);
+}
+
+void* LoadLibraryOrFallback(const base::FilePath& path,
+                            const char* preferred,
+                            const char* fallback) {
+  if (void* library = LoadLibrary(path.Append(preferred))) {
+    return library;
+  }
+  return LoadLibrary(path.Append(fallback));
+}
+
+bool PreferQt6() {
+  auto* cmd = base::CommandLine::ForCurrentProcess();
+  if (cmd->HasSwitch(kQtVersionFlag)) {
+    std::string qt_version_string = cmd->GetSwitchValueASCII(kQtVersionFlag);
+    unsigned int qt_version = 0;
+    if (base::StringToUint(qt_version_string, &qt_version)) {
+      switch (qt_version) {
+        case 5:
+          return false;
+        case 6:
+          return true;
+        default:
+          LOG(ERROR) << "Unsupported QT version " << qt_version;
+      }
+    } else {
+      LOG(ERROR) << "Unable to parse QT version " << qt_version_string;
+    }
+  }
+
+  auto env = base::Environment::Create();
+  auto desktop = base::nix::GetDesktopEnvironment(env.get());
+  return desktop == base::nix::DESKTOP_ENVIRONMENT_KDE6;
+}
+
 int QtWeightToCssWeight(int weight) {
   struct {
     int qt_weight;
@@ -54,7 +103,7 @@ int QtWeightToCssWeight(int weight) {
       {63, 600}, {75, 700}, {81, 800}, {87, 900}, {99, 1000},
   };
 
-  weight = base::clamp(weight, 0, 99);
+  weight = std::clamp(weight, 0, 99);
   for (size_t i = 0; i < std::size(kMapping) - 1; i++) {
     const auto& lo = kMapping[i];
     const auto& hi = kMapping[i + 1];
@@ -96,6 +145,13 @@ class QtNativeTheme : public ui::NativeThemeAura {
   QtNativeTheme& operator=(const QtNativeTheme&) = delete;
   ~QtNativeTheme() override = default;
 
+  void ThemeChanged(bool prefer_dark_theme) {
+    set_use_dark_colors(IsForcedDarkMode() || prefer_dark_theme);
+    set_preferred_color_scheme(CalculatePreferredColorScheme());
+
+    NotifyOnNativeThemeUpdated();
+  }
+
   // ui::NativeTheme:
   DISABLE_CFI_VCALL
   void PaintFrameTopArea(cc::PaintCanvas* canvas,
@@ -125,9 +181,7 @@ class QtNativeTheme : public ui::NativeThemeAura {
 QtUi::QtUi(ui::LinuxUi* fallback_linux_ui)
     : fallback_linux_ui_(fallback_linux_ui) {}
 
-QtUi::~QtUi() {
-  shell_dialog_linux::Finalize();
-}
+QtUi::~QtUi() = default;
 
 std::unique_ptr<ui::LinuxInputMethodContext> QtUi::CreateInputMethodContext(
     ui::LinuxInputMethodContextDelegate* delegate) const {
@@ -145,16 +199,21 @@ void QtUi::GetDefaultFontDescription(std::string* family_out,
                                      int* style_out,
                                      int* weight_out,
                                      gfx::FontRenderParams* params_out) const {
-  if (family_out)
+  if (family_out) {
     *family_out = font_family_;
-  if (size_pixels_out)
+  }
+  if (size_pixels_out) {
     *size_pixels_out = font_size_pixels_;
-  if (style_out)
+  }
+  if (style_out) {
     *style_out = font_style_;
-  if (weight_out)
+  }
+  if (weight_out) {
     *weight_out = font_weight_;
-  if (params_out)
+  }
+  if (params_out) {
     *params_out = font_params_;
+  }
 }
 
 ui::SelectFileDialog* QtUi::CreateSelectFileDialog(
@@ -169,23 +228,53 @@ DISABLE_CFI_DLSYM
 DISABLE_CFI_VCALL
 bool QtUi::Initialize() {
   base::FilePath path;
-  if (!base::PathService::Get(base::DIR_MODULE, &path))
+  if (!base::PathService::Get(base::DIR_MODULE, &path)) {
     return false;
-  path = path.Append("libqt5_shim.so");
-  void* libqt_shim = dlopen(path.value().c_str(), RTLD_NOW | RTLD_GLOBAL);
-  if (!libqt_shim)
+  }
+  void* libqt_shim =
+      PreferQt6()
+          ? LoadLibraryOrFallback(path, "libqt6_shim.so", "libqt5_shim.so")
+          : LoadLibraryOrFallback(path, "libqt5_shim.so", "libqt6_shim.so");
+  if (!libqt_shim) {
     return false;
+  }
   void* create_qt_interface = dlsym(libqt_shim, "CreateQtInterface");
   DCHECK(create_qt_interface);
 
-  cmd_line_ = CopyCmdLine(*base::CommandLine::ForCurrentProcess());
+  // Under certain conditions, a hang may occur in libICE when reading from the
+  // ICE connection.  Chrome doesn't use QT's session save/restore capabilities
+  // and instead manages it's own sessions, so this is not needed anyway.  Unset
+  // SESSION_MANAGER to prevent creating an ICE connection.  See [1] and [2].
+  // [1] https://crbug.com/1450759
+  // [2] https://bugreports.qt.io/browse/QTBUG-38599
+  base::ScopedEnvironmentVariableOverride env_override("SESSION_MANAGER");
+
+  auto cmd_line = *base::CommandLine::ForCurrentProcess();
+  if (auto* delegate = ui::LinuxUiDelegate::GetInstance()) {
+    // Ensure QT is initialized with the same display server protocol as Chrome.
+    // In particular, when running under XWayland, make sure to use the xcb QT
+    // backend instead of the wayland backend.
+    switch (delegate->GetBackend()) {
+      case ui::LinuxUiBackend::kStub:
+        break;
+      case ui::LinuxUiBackend::kX11:
+        cmd_line.AppendArg("-platform");
+        cmd_line.AppendArg("xcb");
+        break;
+      case ui::LinuxUiBackend::kWayland:
+        cmd_line.AppendArg("-platform");
+        cmd_line.AppendArg("wayland");
+        break;
+    }
+  }
+  cmd_line_ = CopyCmdLine(cmd_line);
   shim_.reset((reinterpret_cast<decltype(&CreateQtInterface)>(
       create_qt_interface)(this, &cmd_line_.argc, cmd_line_.argv.data())));
   native_theme_ = std::make_unique<QtNativeTheme>(shim_.get());
   ui::ColorProviderManager::Get().AppendColorProviderInitializer(
       base::BindRepeating(&QtUi::AddNativeColorMixer, base::Unretained(this)));
   FontChanged();
-  shell_dialog_linux::Initialize();
+  ScaleFactorMaybeChangedImpl();
 
   return true;
 }
@@ -196,8 +285,9 @@ ui::NativeTheme* QtUi::GetNativeTheme() const {
 
 bool QtUi::GetColor(int id, SkColor* color, bool use_custom_frame) const {
   auto value = GetColor(id, use_custom_frame);
-  if (value)
+  if (value) {
     *color = *value;
+  }
   return value.has_value();
 }
 
@@ -212,28 +302,28 @@ bool QtUi::GetDisplayProperty(int id, int* result) const {
 }
 
 DISABLE_CFI_VCALL
-SkColor QtUi::GetFocusRingColor() const {
-  return shim_->GetColor(ColorType::kHighlightBg, ColorState::kNormal);
+void QtUi::GetFocusRingColor(SkColor* color) const {
+  *color = shim_->GetColor(ColorType::kHighlightBg, ColorState::kNormal);
 }
 
 DISABLE_CFI_VCALL
-SkColor QtUi::GetActiveSelectionBgColor() const {
-  return shim_->GetColor(ColorType::kHighlightBg, ColorState::kNormal);
+void QtUi::GetActiveSelectionBgColor(SkColor* color) const {
+  *color = shim_->GetColor(ColorType::kHighlightBg, ColorState::kNormal);
 }
 
 DISABLE_CFI_VCALL
-SkColor QtUi::GetActiveSelectionFgColor() const {
-  return shim_->GetColor(ColorType::kHighlightFg, ColorState::kNormal);
+void QtUi::GetActiveSelectionFgColor(SkColor* color) const {
+  *color = shim_->GetColor(ColorType::kHighlightFg, ColorState::kNormal);
 }
 
 DISABLE_CFI_VCALL
-SkColor QtUi::GetInactiveSelectionBgColor() const {
-  return shim_->GetColor(ColorType::kHighlightBg, ColorState::kInactive);
+void QtUi::GetInactiveSelectionBgColor(SkColor* color) const {
+  *color = shim_->GetColor(ColorType::kHighlightBg, ColorState::kInactive);
 }
 
 DISABLE_CFI_VCALL
-SkColor QtUi::GetInactiveSelectionFgColor() const {
-  return shim_->GetColor(ColorType::kHighlightFg, ColorState::kInactive);
+void QtUi::GetInactiveSelectionFgColor(SkColor* color) const {
+  *color = shim_->GetColor(ColorType::kHighlightFg, ColorState::kInactive);
 }
 
 DISABLE_CFI_VCALL
@@ -247,8 +337,9 @@ gfx::Image QtUi::GetIconForContentType(const std::string& content_type,
                                        float scale) const {
   Image image =
       shim_->GetIconForContentType(String(content_type.c_str()), size * scale);
-  if (!image.data_argb.size())
+  if (!image.data_argb.size()) {
     return {};
+  }
 
   SkImageInfo image_info = SkImageInfo::Make(
       image.width, image.height, kBGRA_8888_SkColorType, kPremul_SkAlphaType);
@@ -277,22 +368,36 @@ QtUi::WindowFrameAction QtUi::GetWindowFrameAction(
   }
 }
 
-DISABLE_CFI_VCALL
-float QtUi::GetDeviceScaleFactor() const {
-  return shim_->GetScaleFactor();
-}
-
 DISABLE_CFI_VCALL
 bool QtUi::PreferDarkTheme() const {
   return color_utils::IsDark(
       shim_->GetColor(ColorType::kWindowBg, ColorState::kNormal));
 }
 
+DISABLE_CFI_VCALL
+void QtUi::SetDarkTheme(bool dark) {
+  // Qt::ColorScheme is only available in QT 6.5 and later.
+}
+
 DISABLE_CFI_VCALL
 bool QtUi::AnimationsEnabled() const {
   return shim_->GetAnimationDurationMs() > 0;
 }
 
+void QtUi::AddWindowButtonOrderObserver(
+    ui::WindowButtonOrderObserver* observer) {
+  if (fallback_linux_ui_) {
+    fallback_linux_ui_->AddWindowButtonOrderObserver(observer);
+  }
+}
+
+void QtUi::RemoveWindowButtonOrderObserver(
+    ui::WindowButtonOrderObserver* observer) {
+  if (fallback_linux_ui_) {
+    fallback_linux_ui_->RemoveWindowButtonOrderObserver(observer);
+  }
+}
+
 std::unique_ptr<ui::NavButtonProvider> QtUi::CreateNavButtonProvider() {
   // QT prefers server-side decorations.
   return nullptr;
@@ -346,12 +451,14 @@ void QtUi::FontChanged() {
   auto desc = shim_->GetFontDescription();
 
   font_family_ = desc.family.c_str();
+  // Points are defined at 72 DPI and pixels are 96 DPI by default.
+  constexpr double kPointToPixelRatio = 96.0 / 72.0;
   if (desc.size_pixels > 0) {
     font_size_pixels_ = desc.size_pixels;
-    font_size_points_ = font_size_pixels_ / GetDeviceScaleFactor();
+    font_size_points_ = std::round(font_size_pixels_ / kPointToPixelRatio);
   } else {
     font_size_points_ = desc.size_points;
-    font_size_pixels_ = font_size_points_ * GetDeviceScaleFactor();
+    font_size_pixels_ = std::round(font_size_points_ * kPointToPixelRatio);
   }
   font_style_ = desc.is_italic ? gfx::Font::ITALIC : gfx::Font::NORMAL;
   font_weight_ = QtWeightToCssWeight(desc.weight);
@@ -376,14 +483,27 @@ void QtUi::FontChanged() {
 }
 
 void QtUi::ThemeChanged() {
-  native_theme_->NotifyOnNativeThemeUpdated();
+  native_theme_->ThemeChanged(PreferDarkTheme());
+}
+
+void QtUi::ScaleFactorMaybeChanged() {
+  // This gets called whenever the monitor configuration changes.  Handle the
+  // scale change asynchronously to allow the change to propagate to QT's scale
+  // factor. This also coalesces scale change events together.
+  if (!scale_factor_task_active_) {
+    scale_factor_task_active_ = true;
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+        FROM_HERE, base::BindOnce(&QtUi::ScaleFactorMaybeChangedImpl,
+                                  weak_factory_.GetWeakPtr()));
+  }
 }
 
 DISABLE_CFI_VCALL
 void QtUi::AddNativeColorMixer(ui::ColorProvider* provider,
-                               const ui::ColorProviderManager::Key& key) {
-  if (key.system_theme != ui::SystemTheme::kQt)
+                               const ui::ColorProviderKey& key) {
+  if (key.system_theme != ui::SystemTheme::kQt) {
     return;
+  }
 
   ui::ColorMixer& mixer = provider->AddMixer();
   // These color constants are required by native_chrome_color_mixer_linux.cc
@@ -428,20 +548,27 @@ void QtUi::AddNativeColorMixer(ui::ColorProvider* provider,
       {ui::kColorNativeHeaderSeparatorBorderInactive, ColorType::kMidground,
        ColorState::kInactive},
       {ui::kColorNativeLabelForeground, ColorType::kWindowFg},
-      {ui::kColorNativeTabForegroundInactiveFrameActive, ColorType::kButtonFg},
-      {ui::kColorNativeTabForegroundInactiveFrameInactive, ColorType::kButtonFg,
-       ColorState::kInactive},
       {ui::kColorNativeTextfieldBorderUnfocused, ColorType::kMidground,
        ColorState::kInactive},
       {ui::kColorNativeToolbarBackground, ColorType::kButtonBg},
   };
-  for (const auto& map : kMaps)
+  for (const auto& map : kMaps) {
     mixer[map.id] = {shim_->GetColor(map.role, map.state)};
+  }
 
+  const bool use_custom_frame =
+      key.frame_type == ui::ColorProviderKey::FrameType::kChromium;
   mixer[ui::kColorFrameActive] = {
-      shim_->GetFrameColor(ColorState::kNormal, true)};
+      shim_->GetFrameColor(ColorState::kNormal, use_custom_frame)};
   mixer[ui::kColorFrameInactive] = {
-      shim_->GetFrameColor(ColorState::kInactive, true)};
+      shim_->GetFrameColor(ColorState::kInactive, use_custom_frame)};
+
+  const SkColor button_fg =
+      shim_->GetColor(ColorType::kButtonFg, ColorState::kNormal);
+  mixer[ui::kColorNativeTabForegroundInactiveFrameActive] =
+      ui::BlendForMinContrast({button_fg}, {ui::kColorFrameActive});
+  mixer[ui::kColorNativeTabForegroundInactiveFrameInactive] =
+      ui::BlendForMinContrast({button_fg}, {ui::kColorFrameInactive});
 }
 
 DISABLE_CFI_VCALL
@@ -510,6 +637,30 @@ absl::optional<SkColor> QtUi::GetColor(int id, bool use_custom_frame) const {
   }
 }
 
+DISABLE_CFI_VCALL
+void QtUi::ScaleFactorMaybeChangedImpl() {
+  scale_factor_task_active_ = false;
+  qt::MonitorScale* qt_monitors;
+  ui::DisplayConfig new_config;
+  size_t n_monitors =
+      shim_->GetMonitorConfig(&qt_monitors, &new_config.primary_scale);
+  std::vector<ui::DisplayGeometry> ui_monitors;
+  ui_monitors.reserve(n_monitors);
+  for (size_t i = 0; i < n_monitors; i++) {
+    const qt::MonitorScale& monitor = qt_monitors[i];
+    ui_monitors.push_back(ui::DisplayGeometry{
+        {monitor.x_px, monitor.y_px, monitor.width_px, monitor.height_px},
+        monitor.scale});
+  }
+  if (display_config() != new_config) {
+    display_config() = std::move(new_config);
+    for (ui::DeviceScaleFactorObserver& observer :
+         device_scale_factor_observer_list()) {
+      observer.OnDeviceScaleFactorChanged();
+    }
+  }
+}
+
 std::unique_ptr<ui::LinuxUiAndTheme> CreateQtUi(
     ui::LinuxUi* fallback_linux_ui) {
   return std::make_unique<QtUi>(fallback_linux_ui);
