@@ -12,6 +12,7 @@
 #include "base/memory/raw_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "build/build_config.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "ui/accessibility/ax_enums.mojom-forward.h"
 #include "ui/base/class_property.h"
 #include "ui/base/metadata/metadata_header_macros.h"
@@ -142,7 +143,7 @@ class VIEWS_EXPORT BubbleDialogDelegate : public DialogDelegate {
   // GetAnchorRect() has changed. You only need to do this if you have
   // overridden GetAnchorRect() - if you are using an anchor view or anchor rect
   // normally, do not call this.
-  void OnAnchorBoundsChanged();
+  virtual void OnAnchorBoundsChanged();
 
   // Call this method to update view shown time stamp of underneath input
   // protectors.
@@ -269,6 +270,11 @@ class VIEWS_EXPORT BubbleDialogDelegate : public DialogDelegate {
     title_margins_ = title_margins;
   }
 
+  gfx::Insets footnote_margins() const { return footnote_margins_; }
+  void set_footnote_margins(const gfx::Insets& footnote_margins) {
+    footnote_margins_ = footnote_margins;
+  }
+
   // Sets whether or not CreateClientView() returns a Layer backed ClientView.
   // TODO(pbos): Remove all calls to this, then remove `paint_client_to_layer_`.
   // See comment around `paint_client_to_layer_`.
@@ -390,6 +396,7 @@ class VIEWS_EXPORT BubbleDialogDelegate : public DialogDelegate {
   void SetAnchoredDialogKey();
 
   gfx::Insets title_margins_;
+  gfx::Insets footnote_margins_;
   BubbleBorder::Arrow arrow_ = BubbleBorder::NONE;
   BubbleBorder::Shadow shadow_;
   SkColor color_ = gfx::kPlaceholderColor;
@@ -416,7 +423,7 @@ class VIEWS_EXPORT BubbleDialogDelegate : public DialogDelegate {
   mutable absl::optional<gfx::Rect> anchor_rect_;
 
   bool accept_events_ = true;
-  gfx::NativeView parent_window_ = nullptr;
+  gfx::NativeView parent_window_ = gfx::NativeView();
 
   // By default, all BubbleDialogDelegates have parent windows.
   bool has_parent_ = true;
@@ -445,6 +452,9 @@ class VIEWS_EXPORT BubbleDialogDelegate : public DialogDelegate {
   // monitor clicks as well for the desired behavior.
   std::unique_ptr<ui::BubbleCloser> mac_bubble_closer_;
 #endif
+
+  // Used to ensure the button remains anchored while this dialog is open.
+  absl::optional<Button::ScopedAnchorHighlight> button_anchor_higlight_;
 };
 
 // BubbleDialogDelegateView is a BubbleDialogDelegate that is also a View.
@@ -496,7 +506,7 @@ class VIEWS_EXPORT BubbleDialogDelegateView : public BubbleDialogDelegate,
 };
 
 BEGIN_VIEW_BUILDER(VIEWS_EXPORT, BubbleDialogDelegateView, View)
-VIEW_BUILDER_PROPERTY(ax::mojom::Role, AccessibleRole)
+VIEW_BUILDER_PROPERTY(ax::mojom::Role, AccessibleWindowRole)
 VIEW_BUILDER_PROPERTY(std::u16string, AccessibleTitle)
 VIEW_BUILDER_PROPERTY(bool, CanMaximize)
 VIEW_BUILDER_PROPERTY(bool, CanMinimize)
@@ -505,9 +515,9 @@ VIEW_BUILDER_VIEW_TYPE_PROPERTY(views::View, ExtraView)
 VIEW_BUILDER_VIEW_TYPE_PROPERTY(views::View, FootnoteView)
 VIEW_BUILDER_PROPERTY(bool, FocusTraversesOut)
 VIEW_BUILDER_PROPERTY(bool, EnableArrowKeyTraversal)
-VIEW_BUILDER_PROPERTY(gfx::ImageSkia, Icon)
-VIEW_BUILDER_PROPERTY(gfx::ImageSkia, MainImage)
-VIEW_BUILDER_PROPERTY(gfx::ImageSkia, AppIcon)
+VIEW_BUILDER_PROPERTY(ui::ImageModel, Icon)
+VIEW_BUILDER_PROPERTY(ui::ImageModel, AppIcon)
+VIEW_BUILDER_PROPERTY(ui::ImageModel, MainImage)
 VIEW_BUILDER_PROPERTY(ui::ModalType, ModalType)
 VIEW_BUILDER_PROPERTY(bool, OwnedByWidget)
 VIEW_BUILDER_PROPERTY(bool, ShowCloseButton)
