@@ -16,11 +16,10 @@
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "ui/base/hit_test.h"
-#include "ui/display/test/scoped_screen_override.h"
 #include "ui/display/test/test_screen.h"
 #include "ui/events/event_utils.h"
 #include "ui/views/animation/ink_drop.h"
-#include "ui/views/animation/test/ink_drop_host_view_test_api.h"
+#include "ui/views/animation/test/ink_drop_host_test_api.h"
 #include "ui/views/animation/test/test_ink_drop.h"
 #include "ui/views/bubble/bubble_frame_view.h"
 #include "ui/views/controls/button/button.h"
@@ -37,10 +36,6 @@
 #include "ui/views/widget/widget.h"
 #include "ui/views/widget/widget_observer.h"
 
-#if BUILDFLAG(IS_WIN)
-#include "ui/base/win/shell.h"
-#endif
-
 namespace views {
 
 using test::TestInkDrop;
@@ -113,7 +108,7 @@ class TestAlertBubbleDialogDelegateView : public TestBubbleDialogDelegateView {
  public:
   explicit TestAlertBubbleDialogDelegateView(View* anchor_view)
       : TestBubbleDialogDelegateView(anchor_view) {
-    SetAccessibleRole(ax::mojom::Role::kAlertDialog);
+    SetAccessibleWindowRole(ax::mojom::Role::kAlertDialog);
   }
   ~TestAlertBubbleDialogDelegateView() override = default;
 };
@@ -382,7 +377,7 @@ TEST_F(BubbleDialogDelegateViewTest, NoParentWidget) {
       new TestBubbleDialogDelegateView(nullptr);
   bubble_delegate->set_has_parent(false);
   WidgetAutoclosePtr bubble_widget(
-      BubbleDialogDelegateView::CreateBubble(std::move(bubble_delegate)));
+      BubbleDialogDelegateView::CreateBubble(bubble_delegate));
   EXPECT_EQ(bubble_delegate, bubble_widget->widget_delegate());
   EXPECT_EQ(bubble_widget.get(), bubble_delegate->GetWidget());
   EXPECT_EQ(nullptr, bubble_widget->parent());
@@ -409,21 +404,13 @@ TEST_F(BubbleDialogDelegateViewTest, NonClientHitTest) {
   BubbleDialogDelegateView::CreateBubble(bubble_delegate);
   BubbleFrameView* frame = bubble_delegate->GetBubbleFrameView();
 
-#if BUILDFLAG(IS_WIN)
-  bool is_aero_glass_enabled = ui::win::IsAeroGlassEnabled();
-#endif
-
   struct {
     const int point;
     const int hit;
   } kTestCases[] = {
-#if BUILDFLAG(IS_WIN)
-    {0, is_aero_glass_enabled ? HTTRANSPARENT : HTNOWHERE},
-#else
-    {0, HTTRANSPARENT},
-#endif
-    {60, HTCLIENT},
-    {1000, HTNOWHERE},
+      {0, HTTRANSPARENT},
+      {60, HTCLIENT},
+      {1000, HTNOWHERE},
   };
 
   for (const auto& test_case : kTestCases) {
@@ -622,11 +609,9 @@ TEST_F(BubbleDialogDelegateViewTest, CustomTitle) {
   bubble_frame->InvalidateLayout();
   views::test::RunScheduledLayout(bubble_frame);
 
-  Button* close_button = bubble_frame->GetCloseButtonForTesting();
+  Button* close_button = bubble_frame->close_button();
   // Title moves over for the close button.
-  EXPECT_EQ(close_button->x() - LayoutProvider::Get()->GetDistanceMetric(
-                                    DISTANCE_CLOSE_BUTTON_MARGIN),
-            title_container->bounds().right());
+  EXPECT_GT(close_button->x(), title_container->bounds().right());
 
   LayoutProvider* provider = LayoutProvider::Get();
   const gfx::Insets content_margins = provider->GetDialogInsetsForContentType(
