@@ -16,8 +16,8 @@
 #include "ui/base/models/dialog_model.h"
 #include "ui/base/models/dialog_model_field.h"
 #include "ui/base/ui_base_types.h"
+#include "ui/gfx/color_utils.h"
 #include "ui/gfx/geometry/insets.h"
-#include "ui/views/accessibility/accessibility_paint_checks.h"
 #include "ui/views/accessibility/view_accessibility.h"
 #include "ui/views/border.h"
 #include "ui/views/bubble/bubble_dialog_delegate_view.h"
@@ -170,7 +170,9 @@ class CheckboxControl : public Checkbox {
         DISTANCE_RELATED_LABEL_HORIZONTAL));
     layout->set_cross_axis_alignment(BoxLayout::CrossAxisAlignment::kStart);
 
-    SetAssociatedLabel(label.get());
+    // TODO(accessibility): There is no `SetAccessibilityProperties` which takes
+    // a labelling view to set the accessible name.
+    SetAccessibleName(label.get());
 
     AddChildView(std::move(label));
   }
@@ -248,15 +250,14 @@ class BubbleDialogModelHost::ContentsView : public BoxLayoutView {
         parent_->model_->dark_mode_icon(parent_->GetPassKey());
     if (!dark_mode_icon.IsEmpty() &&
         color_utils::IsDark(parent_->GetBackgroundColor())) {
-      parent_->SetIcon(dark_mode_icon.GetImage().AsImageSkia());
+      parent_->SetIcon(dark_mode_icon);
       return;
     }
-    parent_->SetIcon(
-        parent_->model_->icon(GetPassKey()).GetImage().AsImageSkia());
+    parent_->SetIcon(parent_->model_->icon(GetPassKey()));
   }
 
  private:
-  const raw_ptr<BubbleDialogModelHost> parent_;
+  const raw_ptr<BubbleDialogModelHost, DanglingUntriaged> parent_;
 };
 
 class BubbleDialogModelHost::LayoutConsensusView : public View {
@@ -364,12 +365,12 @@ BubbleDialogModelHost::BubbleDialogModelHost(
   // Dialog callbacks can safely refer to |model_|, they can't be called after
   // Widget::Close() calls WidgetWillClose() synchronously so there shouldn't
   // be any dangling references after model removal.
-  SetAcceptCallback(base::BindOnce(&ui::DialogModel::OnDialogAcceptAction,
-                                   base::Unretained(model_.get()),
-                                   GetPassKey()));
-  SetCancelCallback(base::BindOnce(&ui::DialogModel::OnDialogCancelAction,
-                                   base::Unretained(model_.get()),
-                                   GetPassKey()));
+  SetAcceptCallbackWithClose(
+      base::BindRepeating(&ui::DialogModel::OnDialogAcceptAction,
+                          base::Unretained(model_.get()), GetPassKey()));
+  SetCancelCallbackWithClose(
+      base::BindRepeating(&ui::DialogModel::OnDialogCancelAction,
+                          base::Unretained(model_.get()), GetPassKey()));
   SetCloseCallback(base::BindOnce(&ui::DialogModel::OnDialogCloseAction,
                                   base::Unretained(model_.get()),
                                   GetPassKey()));
@@ -385,16 +386,28 @@ BubbleDialogModelHost::BubbleDialogModelHost(
   auto* ok_button = model_->ok_button(GetPassKey());
   if (ok_button) {
     button_mask |= ui::DIALOG_BUTTON_OK;
-    if (!ok_button->label(GetPassKey()).empty())
+    if (!ok_button->label(GetPassKey()).empty()) {
       SetButtonLabel(ui::DIALOG_BUTTON_OK, ok_button->label(GetPassKey()));
+    }
+    if (ok_button->style(GetPassKey())) {
+      SetButtonStyle(ui::DIALOG_BUTTON_OK, ok_button->style(GetPassKey()));
+    }
+    SetButtonEnabled(ui::DIALOG_BUTTON_OK, ok_button->is_enabled(GetPassKey()));
   }
 
   auto* cancel_button = model_->cancel_button(GetPassKey());
   if (cancel_button) {
     button_mask |= ui::DIALOG_BUTTON_CANCEL;
-    if (!cancel_button->label(GetPassKey()).empty())
+    if (!cancel_button->label(GetPassKey()).empty()) {
       SetButtonLabel(ui::DIALOG_BUTTON_CANCEL,
                      cancel_button->label(GetPassKey()));
+    }
+    if (cancel_button->style(GetPassKey())) {
+      SetButtonStyle(ui::DIALOG_BUTTON_CANCEL,
+                     cancel_button->style(GetPassKey()));
+    }
+    SetButtonEnabled(ui::DIALOG_BUTTON_CANCEL,
+                     cancel_button->is_enabled(GetPassKey()));
   }
 
   // TODO(pbos): Consider refactoring ::SetExtraView() so it can be called after
@@ -403,10 +416,16 @@ BubbleDialogModelHost::BubbleDialogModelHost(
   if (ui::DialogModelButton* extra_button =
           model_->extra_button(GetPassKey())) {
     DCHECK(!model_->extra_link(GetPassKey()));
-    SetExtraView(std::make_unique<MdTextButton>(
-        base::BindRepeating(&ui::DialogModelButton::OnPressed,
-                            base::Unretained(extra_button), GetPassKey()),
-        extra_button->label(GetPassKey())));
+    auto builder = views::Builder<MdTextButton>()
+                       .SetCallback(base::BindRepeating(
+                           &ui::DialogModelButton::OnPressed,
+                           base::Unretained(extra_button), GetPassKey()))
+                       .SetText(extra_button->label(GetPassKey()));
+    if (extra_button->style(GetPassKey())) {
+      builder.SetStyle(extra_button->style(GetPassKey()).value());
+    }
+    builder.SetEnabled(extra_button->is_enabled(GetPassKey()));
+    SetExtraView(std::move(builder).Build());
   } else if (ui::DialogModelLabel::TextReplacement* extra_link =
                  model_->extra_link(GetPassKey())) {
     DCHECK(extra_link->callback().has_value());
@@ -422,6 +441,10 @@ BubbleDialogModelHost::BubbleDialogModelHost(
 
   SetTitle(model_->title(GetPassKey()));
 
+  if (!model_->accessible_title(GetPassKey()).empty()) {
+    SetAccessibleTitle(model_->accessible_title(GetPassKey()));
+  }
+
   SetSubtitle(model_->subtitle(GetPassKey()));
 
   if (!model_->main_image(GetPassKey()).IsEmpty())
@@ -434,8 +457,7 @@ BubbleDialogModelHost::BubbleDialogModelHost(
   }
 
   if (!model_->icon(GetPassKey()).IsEmpty()) {
-    // TODO(pbos): Consider adding ImageModel support to SetIcon().
-    SetIcon(model_->icon(GetPassKey()).GetImage().AsImageSkia());
+    SetIcon(model_->icon(GetPassKey()));
     SetShowIcon(true);
   }
 
@@ -448,9 +470,9 @@ BubbleDialogModelHost::BubbleDialogModelHost(
     // TODO(accessibility): review the role mappings for alerts and dialogs,
     // making sure they are translated to the best candidate in each flatform
     // without resorting to hacks like this.
-    SetAccessibleRole(ax::mojom::Role::kAlert);
+    SetAccessibleWindowRole(ax::mojom::Role::kAlert);
 #else
-    SetAccessibleRole(ax::mojom::Role::kAlertDialog);
+    SetAccessibleWindowRole(ax::mojom::Role::kAlertDialog);
 #endif
   }
 
@@ -566,8 +588,7 @@ void BubbleDialogModelHost::OnFieldAdded(ui::DialogModelField* field) {
   switch (field->type(GetPassKey())) {
     case ui::DialogModelField::kButton:
       // TODO(pbos): Add support for buttons that are part of content area.
-      NOTREACHED();
-      return;
+      NOTREACHED_NORETURN();
     case ui::DialogModelField::kParagraph:
       AddOrUpdateParagraph(field->AsParagraph(GetPassKey()));
       break;
@@ -599,10 +620,26 @@ void BubbleDialogModelHost::OnFieldAdded(ui::DialogModelField* field) {
   }
   UpdateSpacingAndMargins();
 
+  UpdateFieldVisibility(field);
+
   if (GetBubbleFrameView())
     SizeToContents();
 }
 
+void BubbleDialogModelHost::OnFieldChanged(ui::DialogModelField* field) {
+  CHECK(field);
+
+  UpdateFieldVisibility(field);
+
+  if (field->type(GetPassKey()) == ui::DialogModelField::kButton) {
+    UpdateButton(field->AsButton(GetPassKey()));
+  }
+
+  // If the contents of the dialog change (text, field visitiblity, etc.), the
+  // dialog may need to be resized.
+  SizeToContents();
+}
+
 void BubbleDialogModelHost::AddInitialFields() {
   DCHECK(contents_view_->children().empty())
       << "This should only be called once.";
@@ -619,6 +656,15 @@ void BubbleDialogModelHost::UpdateSpacingAndMargins() {
   dialog_side_insets.set_top(0);
   dialog_side_insets.set_bottom(0);
 
+  // If there is a Main Image, the left dialog inset value is no longer the
+  // correct metric. Use the related control metric instead.
+  // TODO(kylixrd): Investigate whether this should be a unique distance metric
+  // or if the related control metric is valid.
+  if (!GetMainImage().IsEmpty()) {
+    dialog_side_insets.set_left(layout_provider->GetDistanceMetric(
+        DISTANCE_RELATED_CONTROL_HORIZONTAL));
+  }
+
   ui::DialogModelField* first_field = nullptr;
   ui::DialogModelField* last_field = nullptr;
 
@@ -670,6 +716,14 @@ void BubbleDialogModelHost::UpdateSpacingAndMargins() {
                                 bottom_margin >= 0 ? bottom_margin : 0, 0));
 }
 
+void BubbleDialogModelHost::UpdateFieldVisibility(ui::DialogModelField* field) {
+  DialogModelHostField host_field = FindDialogModelHostField(field);
+
+  if (host_field.field_view) {
+    host_field.field_view->SetVisible(field->is_visible());
+  }
+}
+
 void BubbleDialogModelHost::OnWindowClosing() {
   // If the model has been removed we have already notified it of closing on the
   // ::Close() stack.
@@ -682,6 +736,7 @@ void BubbleDialogModelHost::OnWindowClosing() {
 void BubbleDialogModelHost::AddOrUpdateParagraph(
     ui::DialogModelParagraph* model_field) {
   // TODO(pbos): Handle updating existing field.
+
   std::unique_ptr<View> view =
       model_field->header(GetPassKey()).empty()
           ? CreateViewForLabel(model_field->label(GetPassKey()))
@@ -797,12 +852,6 @@ void BubbleDialogModelHost::AddOrUpdateTextfield(
   // TODO(pbos): Support updates to the existing model.
 
   auto textfield = std::make_unique<Textfield>();
-  // TODO(crbug.com/1218186): Remove this, this is in place temporarily to be
-  // able to submit accessibility checks, but this focusable View needs to
-  // add a name so that the screen reader knows what to announce. The
-  // placeholder name may need to be pushed into DialogModel, unless we can tie
-  // this to the label. Maybe SetAssociatedField on Textfield is sufficient?
-  textfield->SetProperty(kSkipAccessibilityPaintChecks, true);
   textfield->SetAccessibleName(
       model_field->accessible_name(GetPassKey()).empty()
           ? model_field->label(GetPassKey())
@@ -833,6 +882,21 @@ void BubbleDialogModelHost::AddOrUpdateTextfield(
                           std::move(textfield), font_list);
 }
 
+void BubbleDialogModelHost::UpdateButton(ui::DialogModelButton* model_field) {
+  std::u16string label = model_field->label(GetPassKey());
+  if (model_field == model_->ok_button(GetPassKey())) {
+    SetButtonLabel(ui::DIALOG_BUTTON_OK, label);
+  } else if (model_field == model_->cancel_button(GetPassKey())) {
+    SetButtonLabel(ui::DIALOG_BUTTON_CANCEL, label);
+  } else if (model_field == model_->extra_button(GetPassKey())) {
+    static_cast<MdTextButton*>(
+        GetTargetView(FindDialogModelHostField(model_field)))
+        ->SetText(label);
+  } else {
+    NOTIMPLEMENTED();
+  }
+}
+
 void BubbleDialogModelHost::AddViewForLabelAndField(
     ui::DialogModelField* model_field,
     const std::u16string& label_text,
@@ -901,7 +965,7 @@ BubbleDialogModelHost::FindDialogModelHostField(ui::DialogModelField* field) {
     if (info.dialog_model_field == field)
       return info;
   }
-  NOTREACHED();
+  // TODO(pbos): `field` could correspond to a button.
   return {};
 }
 
@@ -911,14 +975,13 @@ BubbleDialogModelHost::FindDialogModelHostField(View* view) {
     if (info.field_view == view)
       return info;
   }
-  NOTREACHED();
-  return {};
+  NOTREACHED_NORETURN();
 }
 
 View* BubbleDialogModelHost::GetTargetView(
     const DialogModelHostField& field_view_info) {
-  return field_view_info.focusable_view ? field_view_info.focusable_view
-                                        : field_view_info.field_view;
+  return field_view_info.focusable_view ? field_view_info.focusable_view.get()
+                                        : field_view_info.field_view.get();
 }
 
 bool BubbleDialogModelHost::DialogModelLabelRequiresStyledLabel(
