@@ -10,15 +10,21 @@
 #include <utility>
 #include <vector>
 
+#include "base/gtest_prod_util.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "build/build_config.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/menu_separator_types.h"
 #include "ui/base/themed_vector_icon.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/gfx/geometry/insets.h"
+#include "ui/gfx/geometry/size.h"
 #include "ui/gfx/image/image_skia.h"
+#include "ui/strings/grit/ui_strings.h"
+#include "ui/views/controls/menu/menu_config.h"
 #include "ui/views/controls/menu/menu_controller.h"
 #include "ui/views/controls/menu/menu_types.h"
 #include "ui/views/view.h"
@@ -37,17 +43,13 @@ namespace internal {
 class MenuRunnerImpl;
 }
 
-namespace test {
-class TestMenuItemViewShown;
-class TestMenuItemViewNotShown;
-}  // namespace test
-
 class ImageView;
 class MenuController;
+class MenuControllerTest;
 class MenuDelegate;
 class Separator;
-class TestMenuItemView;
 class SubmenuView;
+class TestMenuItemView;
 
 // MenuItemView --------------------------------------------------------------
 
@@ -75,14 +77,6 @@ class VIEWS_EXPORT MenuItemView : public View {
  public:
   METADATA_HEADER(MenuItemView);
 
-  friend class MenuController;
-
-  // ID used to identify menu items.
-  static const int kMenuItemViewID;
-
-  // ID used to identify empty menu items.
-  static const int kEmptyMenuItemViewID;
-
   // Different types of menu items.
   enum class Type {
     kNormal,             // Performs an action when selected.
@@ -119,6 +113,15 @@ class VIEWS_EXPORT MenuItemView : public View {
     int height = 0;
   };
 
+  // The data structure which is used to paint a background on the menu item.
+  struct MenuItemBackground {
+    MenuItemBackground(ui::ColorId background_color_id, int corner_radius)
+        : background_color_id(background_color_id),
+          corner_radius(corner_radius) {}
+    ui::ColorId background_color_id;
+    int corner_radius = 0;
+  };
+
   // Constructor for use with the top level menu item. This menu is never
   // shown to the user, rather its use as the parent for all menu items.
   explicit MenuItemView(MenuDelegate* delegate = nullptr);
@@ -126,19 +129,14 @@ class VIEWS_EXPORT MenuItemView : public View {
   MenuItemView(const MenuItemView&) = delete;
   MenuItemView& operator=(const MenuItemView&) = delete;
 
+  ~MenuItemView() override;
+
   // Overridden from View:
   std::u16string GetTooltipText(const gfx::Point& p) const override;
   void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
   bool HandleAccessibleAction(const ui::AXActionData& action_data) override;
   FocusBehavior GetFocusBehavior() const override;
 
-  // Returns the preferred height of menu items. This is only valid when the
-  // menu is about to be shown.
-  static int pref_menu_height() { return pref_menu_height_; }
-
-  // X-coordinate of where the label starts.
-  static int label_start() { return label_start_; }
-
   // Returns if a given |anchor| is a bubble or not.
   static bool IsBubble(MenuAnchorPosition anchor);
 
@@ -154,15 +152,42 @@ class VIEWS_EXPORT MenuItemView : public View {
 
   // Add an item to the menu at a specified index.  ChildrenChanged() should
   // called after adding menu items if the menu may be active.
-  MenuItemView* AddMenuItemAt(size_t index,
-                              int item_id,
-                              const std::u16string& label,
-                              const std::u16string& secondary_label,
-                              const std::u16string& minor_text,
-                              const ui::ImageModel& minor_icon,
-                              const ui::ImageModel& icon,
-                              Type type,
-                              ui::MenuSeparatorType separator_style);
+  MenuItemView* AddMenuItemAt(
+      size_t index,
+      int item_id,
+      const std::u16string& label,
+      const std::u16string& secondary_label,
+      const std::u16string& minor_text,
+      const ui::ImageModel& minor_icon,
+      const ui::ImageModel& icon,
+      Type type,
+      ui::MenuSeparatorType separator_style,
+      absl::optional<ui::ColorId> submenu_background_color = absl::nullopt,
+      absl::optional<ui::ColorId> foreground_color = absl::nullopt,
+      absl::optional<ui::ColorId> selected_color_id = absl::nullopt);
+
+  void SetMenuItemBackground(
+      absl::optional<MenuItemBackground> menu_item_background) {
+    menu_item_background_ = menu_item_background;
+  }
+
+  absl::optional<MenuItemBackground> GetMenuItemBackground() {
+    return menu_item_background_;
+  }
+
+  void SetSelectedColorId(absl::optional<ui::ColorId> selected_color_id) {
+    selected_color_id_ = selected_color_id;
+  }
+
+  absl::optional<ui::ColorId> GetSelectedColorId() {
+    return selected_color_id_;
+  }
+
+  void SetHighlightWhenSelectedWithChildViews(
+      bool highlight_when_selected_with_child_views) {
+    highlight_when_selected_with_child_views_ =
+        highlight_when_selected_with_child_views;
+  }
 
   // Remove the specified item from the menu. |item| will be deleted when
   // ChildrenChanged() is invoked.
@@ -184,6 +209,8 @@ class VIEWS_EXPORT MenuItemView : public View {
                                const std::u16string& label = std::u16string(),
                                const ui::ImageModel& icon = ui::ImageModel());
 
+  MenuItemView* AppendTitle(const std::u16string& label);
+
   // Append a submenu to this menu.
   // The returned pointer is owned by this menu.
   MenuItemView* AppendSubMenu(int item_id,
@@ -268,6 +295,9 @@ class VIEWS_EXPORT MenuItemView : public View {
   // SetIcon(). MenuItemView takes ownership of |icon_view|.
   void SetIconView(std::unique_ptr<ImageView> icon_view);
 
+  // Returns the preferred size of the icon view if any, or gfx::Size() if none.
+  gfx::Size GetIconPreferredSize() const;
+
   // Sets the command id of this menu item.
   void SetCommand(int command) { command_ = command; }
 
@@ -282,11 +312,6 @@ class VIEWS_EXPORT MenuItemView : public View {
   }
   bool may_have_mnemonics() const { return may_have_mnemonics_; }
 
-  void set_accessible_name(std::u16string accessible_name) {
-    accessible_name_ = std::move(accessible_name);
-  }
-  const std::u16string& accessible_name() const { return accessible_name_; }
-
   // Called when the drop or selection status (as determined by SubmenuView) may
   // have changed.
   void OnDropOrSelectionStatusMayHaveChanged();
@@ -308,7 +333,16 @@ class VIEWS_EXPORT MenuItemView : public View {
   // Return the preferred dimensions of the item in pixel.
   const MenuItemDimensions& GetDimensions() const;
 
-  // Returns the object responsible for controlling showing the menu.
+  // Returns the earliest horizontal position where content may appear.
+  int GetContentStart() const;
+
+  void set_controller(MenuController* controller) {
+    if (controller) {
+      controller_ = controller->AsWeakPtr();
+    } else {
+      controller_.reset();
+    }
+  }
   MenuController* GetMenuController();
   const MenuController* GetMenuController() const;
 
@@ -325,11 +359,6 @@ class VIEWS_EXPORT MenuItemView : public View {
   // doesn't have a mnemonic.
   char16_t GetMnemonic();
 
-  // Do we have icons? This only has effect on the top menu. Turning this on
-  // makes the menus slightly wider and taller.
-  void set_has_icons(bool has_icons) { has_icons_ = has_icons; }
-  bool has_icons() const { return has_icons_; }
-
   // Returns the descendant with the specified command.
   MenuItemView* GetMenuItemByID(int id);
 
@@ -344,13 +373,13 @@ class VIEWS_EXPORT MenuItemView : public View {
   // item.
   bool has_mnemonics() const { return has_mnemonics_; }
 
-  // Set top and bottom margins in pixels.  If no margin is set or a
-  // negative margin is specified then MenuConfig values are used.
-  void SetMargins(int top_margin, int bottom_margin);
+  void set_vertical_margin(int vertical_margin) {
+    vertical_margin_ = vertical_margin;
+    invalidate_dimensions();
+  }
 
-  // Suppress the right margin if this is set to false.
-  void set_use_right_margin(bool use_right_margin) {
-    use_right_margin_ = use_right_margin;
+  void set_children_use_full_width(bool children_use_full_width) {
+    children_use_full_width_ = children_use_full_width;
   }
 
   // Controls whether this menu has a forced visual selection state. This is
@@ -376,17 +405,19 @@ class VIEWS_EXPORT MenuItemView : public View {
   // item.
   bool IsTraversableByKeyboard() const;
 
+  // Returns the corresponding border padding from the `MenuConfig`.
+  int GetItemHorizontalBorder() const;
+
   bool last_paint_as_selected_for_testing() const {
     return last_paint_as_selected_;
   }
 
+  static std::u16string GetNewBadgeAccessibleDescription();
+
  protected:
   // Creates a MenuItemView. This is used by the various AddXXX methods.
   MenuItemView(MenuItemView* parent, int command, Type type);
 
-  // MenuRunner owns MenuItemView and should be the only one deleting it.
-  ~MenuItemView() override;
-
   // View:
   void ChildPreferredSizeChanged(View* child) override;
   void OnThemeChanged() override;
@@ -401,10 +432,11 @@ class VIEWS_EXPORT MenuItemView : public View {
   int GetBottomMargin() const;
 
  private:
-  friend class internal::MenuRunnerImpl;        // For access to ~MenuItemView.
-  friend class test::TestMenuItemViewShown;     // for access to |submenu_|;
-  friend class test::TestMenuItemViewNotShown;  // for access to |submenu_|;
-  friend class TestMenuItemView;  // For access to AddEmptyMenus();
+  friend class MenuController;
+  friend class internal::MenuRunnerImpl;
+  friend class MenuControllerTest;
+  friend class TestMenuItemView;
+  FRIEND_TEST_ALL_PREFIXES(MenuControllerTest, RepostEventToEmptyMenuItem);
 
   enum class PaintMode { kNormal, kForDrag };
 
@@ -420,31 +452,24 @@ class VIEWS_EXPORT MenuItemView : public View {
                Type type,
                MenuDelegate* delegate);
 
-  // Calculates all sizes that we can from the OS.
-  //
-  // This is invoked prior to Running a menu.
-  void UpdateMenuPartSizes();
+  const SubmenuView* GetContainingSubmenu() const {
+    return parent_menu_item_->GetSubmenu();
+  }
 
   // The RunXXX methods call into this to set up the necessary state before
-  // running. |is_first_menu| is true if no menus are currently showing.
-  void PrepareForRun(bool is_first_menu,
-                     bool has_mnemonics,
-                     bool show_mnemonics);
+  // running.
+  void PrepareForRun(bool has_mnemonics, bool show_mnemonics);
 
   // Returns the flags passed to DrawStringRect.
-  int GetDrawStringFlags();
+  int GetDrawStringFlags() const;
 
   // Returns the font list and font color to use for menu text.
   const gfx::FontList GetFontList() const;
   const absl::optional<SkColor> GetMenuLabelColor() const;
 
-  // If this menu item has no children a child is added showing it has no
-  // children. Otherwise AddEmtpyMenus is recursively invoked on child menu
-  // items that have children.
-  void AddEmptyMenus();
-
-  // Undoes the work of AddEmptyMenus.
-  void RemoveEmptyMenus();
+  // Ensures the submenu has an empty menu item iff it needs one, then updates
+  // its metrics.
+  void UpdateEmptyMenusAndMetrics();
 
   // Given bounds within our View, this helper routine mirrors the bounds if
   // necessary.
@@ -483,8 +508,8 @@ class VIEWS_EXPORT MenuItemView : public View {
   // Returns the colors used in painting.
   Colors CalculateColors(bool paint_as_selected) const;
 
-  // Returns the accessible name for this menu item.
-  std::u16string GetAccessibleName() const;
+  // Calculates and returns the accessible name for this menu item.
+  std::u16string CalculateAccessibleName() const;
 
   // Calculates and returns the MenuItemDimensions.
   MenuItemDimensions CalculateDimensions() const;
@@ -497,6 +522,10 @@ class VIEWS_EXPORT MenuItemView : public View {
   //    ApplyMinimumDimensions(x).height >= x.height
   void ApplyMinimumDimensions(MenuItemDimensions* dims) const;
 
+  // Given a proposed `height` for this item, returns the height after ensuring
+  // it reserves sufficient icon height.
+  int ApplyMinIconHeight(int height) const;
+
   // Get the horizontal position at which to draw the menu item's label.
   int GetLabelStartForThisItem() const;
 
@@ -506,12 +535,6 @@ class VIEWS_EXPORT MenuItemView : public View {
   void set_actual_menu_position(MenuPosition actual_menu_position) {
     actual_menu_position_ = actual_menu_position;
   }
-  void set_controller(MenuController* controller) {
-    if (controller)
-      controller_ = controller->AsWeakPtr();
-    else
-      controller_.reset();
-  }
 
   // Returns true if this MenuItemView contains a single child
   // that is responsible for rendering the content.
@@ -524,12 +547,6 @@ class VIEWS_EXPORT MenuItemView : public View {
   // Returns number of child views excluding icon_view.
   int NonIconChildViewsCount() const;
 
-  // Returns the max icon width; recurses over submenus.
-  int GetMaxIconViewWidth() const;
-
-  // Returns true if the menu has items with a checkbox or a radio button.
-  bool HasChecksOrRadioButtons() const;
-
   void invalidate_dimensions() { dimensions_.height = 0; }
   bool is_dimensions_valid() const { return dimensions_.height > 0; }
 
@@ -551,10 +568,18 @@ class VIEWS_EXPORT MenuItemView : public View {
   // could interact with model state.
   bool IsScheduledForDeletion() const;
 
+  void SetForegroundColorId(absl::optional<ui::ColorId> foreground_color_id) {
+    foreground_color_id_ = foreground_color_id;
+  }
+
+  // Returns the corresponding margin from the `MenuConfig` if
+  // `vertical_margin_` is not set.
+  int GetVerticalMargin() const;
+
   // The delegate. This is only valid for the root menu item. You shouldn't
   // use this directly, instead use GetDelegate() which walks the tree as
   // as necessary.
-  raw_ptr<MenuDelegate> delegate_ = nullptr;
+  raw_ptr<MenuDelegate, DanglingUntriaged> delegate_ = nullptr;
 
   // The controller for the run operation, or NULL if the menu isn't showing.
   base::WeakPtr<MenuController> controller_;
@@ -563,7 +588,7 @@ class VIEWS_EXPORT MenuItemView : public View {
   bool canceled_ = false;
 
   // Our parent.
-  raw_ptr<MenuItemView> parent_menu_item_ = nullptr;
+  raw_ptr<MenuItemView, DanglingUntriaged> parent_menu_item_ = nullptr;
 
   // Type of menu. NOTE: MenuItemView doesn't itself represent SEPARATOR,
   // that is handled by an entirely different view class.
@@ -588,13 +613,12 @@ class VIEWS_EXPORT MenuItemView : public View {
   bool may_have_mnemonics_ = true;
 
   // Submenu, created via CreateSubmenu.
-  raw_ptr<SubmenuView> submenu_ = nullptr;
+  raw_ptr<SubmenuView, DanglingUntriaged> submenu_ = nullptr;
 
   std::u16string title_;
   std::u16string secondary_title_;
   std::u16string minor_text_;
   ui::ImageModel minor_icon_;
-  std::u16string accessible_name_;
 
   // Does the title have a mnemonic? Only useful on the root menu item.
   bool has_mnemonics_ = false;
@@ -603,27 +627,12 @@ class VIEWS_EXPORT MenuItemView : public View {
   // MenuConfig says mnemonics should be shown. Only used on the root menu item.
   bool show_mnemonics_ = false;
 
-  // Set if menu has icons or icon_views (applies to root menu item only).
-  bool has_icons_ = false;
-
   // Pointer to a view with a menu icon.
-  raw_ptr<ImageView> icon_view_ = nullptr;
+  raw_ptr<ImageView, DanglingUntriaged> icon_view_ = nullptr;
 
   // The tooltip to show on hover for this menu item.
   std::u16string tooltip_;
 
-  // Width of a menu icon area.
-  static int icon_area_width_;
-
-  // X-coordinate of where the label starts.
-  static int label_start_;
-
-  // Margins between the right of the item and the label.
-  static int item_right_margin_;
-
-  // Preferred height of menu items. Reset every time a menu is run.
-  static int pref_menu_height_;
-
   // Cached dimensions. This is cached as text sizing calculations are quite
   // costly.
   mutable MenuItemDimensions dimensions_;
@@ -631,9 +640,7 @@ class VIEWS_EXPORT MenuItemView : public View {
   // Removed items to be deleted in ChildrenChanged().
   std::vector<View*> removed_items_;
 
-  // Margins in pixels.
-  int top_margin_ = -1;
-  int bottom_margin_ = -1;
+  absl::optional<int> vertical_margin_;
 
   // Corner radius in pixels, for HIGHLIGHTED items placed at the end of a menu.
   int corner_radius_ = 0;
@@ -644,26 +651,33 @@ class VIEWS_EXPORT MenuItemView : public View {
   MenuPosition requested_menu_position_ = MenuPosition::kBestFit;
   MenuPosition actual_menu_position_ = MenuPosition::kBestFit;
 
-  // If set to false, the right margin will be removed for menu lines
-  // containing other elements.
-  bool use_right_margin_ = true;
+  // If set to true, children beyond the normal icon/labels/arrow will be laid
+  // out taking the full width of the menu, instead of stopping at any arrow
+  // column.
+  bool children_use_full_width_ = false;
 
   // Contains an image for the checkbox or radio icon.
-  raw_ptr<ImageView> radio_check_image_view_ = nullptr;
+  raw_ptr<ImageView, DanglingUntriaged> radio_check_image_view_ = nullptr;
 
   // The submenu indicator arrow icon in case the menu item has a Submenu.
-  raw_ptr<ImageView> submenu_arrow_image_view_ = nullptr;
+  raw_ptr<ImageView, DanglingUntriaged> submenu_arrow_image_view_ = nullptr;
 
   // The forced visual selection state of this item, if any.
   absl::optional<bool> forced_visual_selection_;
 
   // The vertical separator that separates the actionable and submenu regions of
   // an ACTIONABLE_SUBMENU.
-  raw_ptr<Separator> vertical_separator_ = nullptr;
+  raw_ptr<Separator, DanglingUntriaged> vertical_separator_ = nullptr;
 
   // Whether this menu item is rendered differently to draw attention to it.
   bool is_alerted_ = false;
 
+  // Legacy implementation for menu items is that if a MenuItemView has a child
+  // view then the item will not be highlighted when selected. This new boolean
+  // will control whether or not the MenuItemView is highlighted when there are
+  // child views.
+  bool highlight_when_selected_with_child_views_ = false;
+
   // If true, ViewHierarchyChanged() will call
   // UpdateSelectionBasedStateIfChanged().
   // UpdateSelectionBasedStateIfChanged() calls to NonIconChildViewsCount().
@@ -676,6 +690,28 @@ class VIEWS_EXPORT MenuItemView : public View {
   // `update_selection_based_state_in_view_herarchy_changed_` is set to false
   // and SetIconView() explicitly calls UpdateSelectionBasedStateIfChanged().
   bool update_selection_based_state_in_view_herarchy_changed_ = true;
+
+  const std::u16string new_badge_text_ = l10n_util::GetStringUTF16(
+      features::IsChromeRefresh2023() ? IDS_NEW_BADGE_UPPERCASE
+                                      : IDS_NEW_BADGE);
+
+  absl::optional<ui::ColorId> foreground_color_id_;
+  absl::optional<MenuItemBackground> menu_item_background_;
+  absl::optional<ui::ColorId> selected_color_id_;
+};
+
+// EmptyMenuMenuItem ----------------------------------------------------------
+
+// EmptyMenuMenuItem is used when a menu has no menu items.
+
+class VIEWS_EXPORT EmptyMenuMenuItem : public MenuItemView {
+ public:
+  METADATA_HEADER(EmptyMenuMenuItem);
+
+  explicit EmptyMenuMenuItem(MenuItemView* parent);
+  EmptyMenuMenuItem(const EmptyMenuMenuItem&) = delete;
+  EmptyMenuMenuItem& operator=(const EmptyMenuMenuItem&) = delete;
+  ~EmptyMenuMenuItem() override = default;
 };
 
 }  // namespace views
