@@ -27,6 +27,7 @@
 #include "ui/views/vector_icons.h"
 #include "ui/views/view_class_properties.h"
 #include "ui/views/view_test_api.h"
+#include "ui/views/view_utils.h"
 
 namespace views {
 
@@ -73,7 +74,7 @@ TEST_F(MenuItemViewUnitTest, TestMenuItemViewWithFlexibleWidthChild) {
   views::MenuItemView* flexible_view = root_menu.AppendMenuItem(2);
   flexible_view->AddChildView(new SquareView());
   // Set margins to 0 so that we know width should match height.
-  flexible_view->SetMargins(0, 0);
+  flexible_view->set_vertical_margin(0);
 
   views::SubmenuView* submenu = root_menu.GetSubmenu();
 
@@ -92,14 +93,15 @@ TEST_F(MenuItemViewUnitTest, TestMenuItemViewWithFlexibleWidthChild) {
   EXPECT_EQ(label_size.width(), flex_height);
 
   // The submenu should be tall enough to allow for both menu items at the
-  // given width.
-  EXPECT_EQ(label_size.height() + flex_height,
-            submenu->GetPreferredSize().height());
+  // given width. (It may be taller if there is padding between/around the
+  // items.)
+  EXPECT_GE(submenu->GetPreferredSize().height(),
+            label_size.height() + flex_height);
 }
 
-// Tests that the top-level menu item with hidden children should contain the
-// "(empty)" menu item to display.
-TEST_F(MenuItemViewUnitTest, TestEmptyTopLevelWhenAllItemsAreHidden) {
+// Tests that a menu item with hidden children should contain the "(empty)" menu
+// item to display.
+TEST_F(MenuItemViewUnitTest, TestEmptyWhenAllItemsAreHidden) {
   views::TestMenuItemView root_menu;
   views::MenuItemView* item1 = root_menu.AppendMenuItem(1, u"item 1");
   views::MenuItemView* item2 = root_menu.AppendMenuItem(2, u"item 2");
@@ -114,49 +116,14 @@ TEST_F(MenuItemViewUnitTest, TestEmptyTopLevelWhenAllItemsAreHidden) {
   EXPECT_EQ(2u, submenu->children().size());
 
   // Adds any empty menu items to the menu, if needed.
-  root_menu.AddEmptyMenus();
+  root_menu.UpdateEmptyMenusAndMetrics();
 
   // Because all of the submenu's children are hidden, an empty menu item should
   // have been added.
   ASSERT_EQ(3u, submenu->children().size());
-  auto* empty_item = static_cast<MenuItemView*>(submenu->children().front());
+  const auto* empty_item =
+      AsViewClass<EmptyMenuMenuItem>(submenu->children().front());
   ASSERT_TRUE(empty_item);
-  ASSERT_EQ(MenuItemView::kEmptyMenuItemViewID, empty_item->GetID());
-  EXPECT_EQ(l10n_util::GetStringUTF16(IDS_APP_MENU_EMPTY_SUBMENU),
-            empty_item->title());
-}
-
-// Tests that submenu with hidden children should contain the "(empty)" menu
-// item to display.
-TEST_F(MenuItemViewUnitTest, TestEmptySubmenuWhenAllChildItemsAreHidden) {
-  views::TestMenuItemView root_menu;
-  MenuItemView* submenu_item = root_menu.AppendSubMenu(1, u"My Submenu");
-  MenuItemView* child1 = submenu_item->AppendMenuItem(1, u"submenu item 1");
-  MenuItemView* child2 = submenu_item->AppendMenuItem(2, u"submenu item 2");
-
-  // Set submenu children to hidden.
-  child1->SetVisible(false);
-  child2->SetVisible(false);
-
-  SubmenuView* submenu = submenu_item->GetSubmenu();
-  ASSERT_TRUE(submenu);
-
-  EXPECT_EQ(2u, submenu->children().size());
-
-  // Adds any empty menu items to the menu, if needed.
-  EXPECT_FALSE(submenu->HasEmptyMenuItemView());
-  root_menu.AddEmptyMenus();
-  EXPECT_TRUE(submenu->HasEmptyMenuItemView());
-  // Because all of the submenu's children are hidden, an empty menu item should
-  // have been added.
-  ASSERT_EQ(3u, submenu->children().size());
-  auto* empty_item = static_cast<MenuItemView*>(submenu->children().front());
-  ASSERT_TRUE(empty_item);
-  // Not allowed to add an duplicated empty menu item
-  // if it already has an empty menu item.
-  root_menu.AddEmptyMenus();
-  ASSERT_EQ(3u, submenu->children().size());
-  ASSERT_EQ(MenuItemView::kEmptyMenuItemViewID, empty_item->GetID());
   EXPECT_EQ(l10n_util::GetStringUTF16(IDS_APP_MENU_EMPTY_SUBMENU),
             empty_item->title());
 }
@@ -251,8 +218,8 @@ TEST_F(TouchableMenuItemViewTest, MinAndMaxWidth) {
   // Test a title which is between the min and max allowed widths.
   gfx::Size item2_size =
       AppendItemAndGetSize(2, u"Item2 bigger than min less than max");
-  EXPECT_GT(item2_size.width(), min_menu_width);
-  EXPECT_LT(item2_size.width(), max_menu_width);
+  EXPECT_GE(item2_size.width(), min_menu_width);
+  EXPECT_LE(item2_size.width(), max_menu_width);
 
   // Test a title which is longer than the max touchable menu width.
   gfx::Size item3_size =
@@ -407,7 +374,7 @@ class MenuItemViewPaintUnitTest : public ViewsTestBase {
 
  private:
   // Owned by MenuRunner.
-  raw_ptr<MenuItemView> menu_item_view_;
+  raw_ptr<MenuItemView, DanglingUntriaged> menu_item_view_;
 
   std::unique_ptr<test::TestMenuDelegate> menu_delegate_;
   std::unique_ptr<MenuRunner> menu_runner_;
@@ -455,6 +422,47 @@ TEST_F(MenuItemViewPaintUnitTest, MinorTextAndIconAssertionCoverage) {
       PaintInfo::CreateRootPaintInfo(canvas_painter.context(), size));
 }
 
+// Provides assertion coverage for painting with custom colors.
+// icons.
+TEST_F(MenuItemViewPaintUnitTest, CustomColorAssertionCoverage) {
+  auto AddItem = [this](auto label, auto submenu_background_color,
+                        auto foreground_color, auto selected_color) {
+    menu_item_view()->AddMenuItemAt(
+        0, 1000, label, std::u16string(), std::u16string(), ui::ImageModel(),
+        ui::ImageModel(), views::MenuItemView::Type::kNormal,
+        ui::NORMAL_SEPARATOR, submenu_background_color, foreground_color,
+        selected_color);
+  };
+  ui::ColorId background_color = ui::kColorComboboxBackground;
+  ui::ColorId foreground_color = ui::kColorDropdownForeground;
+  ui::ColorId selected_color = ui::kColorMenuItemForegroundHighlighted;
+  AddItem(u"No custom colors", absl::nullopt, absl::nullopt, absl::nullopt);
+  AddItem(u"No selected color", background_color, foreground_color,
+          absl::nullopt);
+  AddItem(u"No foreground color", background_color, absl::nullopt,
+          selected_color);
+  AddItem(u"No background color", absl::nullopt, foreground_color,
+          selected_color);
+  AddItem(u"No background or foreground", absl::nullopt, absl::nullopt,
+          selected_color);
+  AddItem(u"No background or selected", absl::nullopt, foreground_color,
+          absl::nullopt);
+  AddItem(u"No foreground or selected", background_color, absl::nullopt,
+          absl::nullopt);
+  AddItem(u"All colors", background_color, foreground_color, selected_color);
+
+  menu_runner()->RunMenuAt(widget(), nullptr, gfx::Rect(),
+                           MenuAnchorPosition::kTopLeft,
+                           ui::MENU_SOURCE_KEYBOARD);
+
+  SkBitmap bitmap;
+  gfx::Size size = menu_item_view()->GetMirroredBounds().size();
+  ui::CanvasPainter canvas_painter(&bitmap, size, 1.f, SK_ColorTRANSPARENT,
+                                   false);
+  menu_item_view()->GetSubmenu()->Paint(
+      PaintInfo::CreateRootPaintInfo(canvas_painter.context(), size));
+}
+
 // Verifies a call to MenuItemView::OnPaint() doesn't trigger a call to
 // MenuItemView::submenu_arrow_image_view_::SchedulePaint(). This is a
 // regression test for https://crbug.com/1245854.
@@ -607,11 +615,6 @@ class MenuItemViewAccessTest : public MenuItemViewPaintUnitTest {
  private:
   class DisallowMenuDelegate : public test::TestMenuDelegate {
    public:
-    const gfx::FontList* GetLabelFontList(int command_id) const override {
-      EXPECT_NE(1, command_id);
-      return nullptr;
-    }
-
     absl::optional<SkColor> GetLabelColor(int command_id) const override {
       EXPECT_NE(1, command_id);
       return absl::nullopt;
