@@ -6,14 +6,14 @@
 #define UI_VIEWS_EXAMPLES_EXAMPLES_COLOR_MIXER_H_
 
 #include "ui/color/color_provider.h"
-#include "ui/color/color_provider_manager.h"
+#include "ui/color/color_provider_key.h"
 #include "ui/views/examples/views_examples_export.h"
 
 namespace views::examples {
 
 VIEWS_EXAMPLES_EXPORT void AddExamplesColorMixers(
     ui::ColorProvider* color_provider,
-    const ui::ColorProviderManager::Key& key);
+    const ui::ColorProviderKey& key);
 
 }  // namespace views::examples
 
