@@ -5,8 +5,7 @@
 #ifndef UI_VIEWS_EXAMPLES_EXAMPLES_EXIT_CODE_H_
 #define UI_VIEWS_EXAMPLES_EXAMPLES_EXIT_CODE_H_
 
-namespace views {
-namespace examples {
+namespace views::examples {
 
 enum class ExamplesExitCode {
   // Comparison succeeded.
@@ -19,7 +18,6 @@ enum class ExamplesExitCode {
   kNone,
 };
 
-}  // namespace examples
-}  // namespace views
+}  // namespace views::examples
 
 #endif  // UI_VIEWS_EXAMPLES_EXAMPLES_EXIT_CODE_H_
