@@ -12,6 +12,7 @@
 
 #include "base/command_line.h"
 #include "base/memory/raw_ptr.h"
+#include "base/memory/scoped_refptr.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/icu_test_util.h"
 #include "build/build_config.h"
@@ -48,10 +49,19 @@ struct FocusTestEvent {
   FocusManager::FocusChangeReason focus_change_reason;
 };
 
+class FocusTestEventList : public base::RefCounted<FocusTestEventList> {
+ public:
+  std::vector<FocusTestEvent> vec;
+
+ private:
+  friend class base::RefCounted<FocusTestEventList>;
+  ~FocusTestEventList() = default;
+};
+
 class SimpleTestView : public View {
  public:
-  SimpleTestView(std::vector<FocusTestEvent>* event_list, int view_id)
-      : event_list_(event_list) {
+  SimpleTestView(scoped_refptr<FocusTestEventList> event_list, int view_id)
+      : event_list_(std::move(event_list)) {
     SetFocusBehavior(FocusBehavior::ALWAYS);
     set_suppress_default_focus_handling();
     SetID(view_id);
@@ -61,7 +71,7 @@ class SimpleTestView : public View {
   SimpleTestView& operator=(const SimpleTestView&) = delete;
 
   void OnFocus() override {
-    event_list_->push_back({
+    event_list_->vec.push_back({
         ON_FOCUS,
         GetID(),
         GetFocusManager()->focus_change_reason(),
@@ -69,7 +79,7 @@ class SimpleTestView : public View {
   }
 
   void OnBlur() override {
-    event_list_->push_back({
+    event_list_->vec.push_back({
         ON_BLUR,
         GetID(),
         GetFocusManager()->focus_change_reason(),
@@ -77,47 +87,46 @@ class SimpleTestView : public View {
   }
 
  private:
-  raw_ptr<std::vector<FocusTestEvent>> event_list_;
+  const scoped_refptr<FocusTestEventList> event_list_;
 };
 
 // Tests that the appropriate Focus related methods are called when a View
 // gets/loses focus.
 TEST_F(FocusManagerTest, ViewFocusCallbacks) {
-  std::vector<FocusTestEvent> event_list;
+  auto event_list = base::MakeRefCounted<FocusTestEventList>();
   const int kView1ID = 1;
   const int kView2ID = 2;
-
-  SimpleTestView* view1 = new SimpleTestView(&event_list, kView1ID);
-  SimpleTestView* view2 = new SimpleTestView(&event_list, kView2ID);
+  SimpleTestView* view1 = new SimpleTestView(event_list, kView1ID);
+  SimpleTestView* view2 = new SimpleTestView(event_list, kView2ID);
   GetContentsView()->AddChildView(view1);
   GetContentsView()->AddChildView(view2);
 
   view1->RequestFocus();
-  ASSERT_EQ(1, static_cast<int>(event_list.size()));
-  EXPECT_EQ(ON_FOCUS, event_list[0].type);
-  EXPECT_EQ(kView1ID, event_list[0].view_id);
+  ASSERT_EQ(1, static_cast<int>(event_list->vec.size()));
+  EXPECT_EQ(ON_FOCUS, event_list->vec[0].type);
+  EXPECT_EQ(kView1ID, event_list->vec[0].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[0].focus_change_reason);
+            event_list->vec[0].focus_change_reason);
 
-  event_list.clear();
+  event_list->vec.clear();
   view2->RequestFocus();
-  ASSERT_EQ(2, static_cast<int>(event_list.size()));
-  EXPECT_EQ(ON_BLUR, event_list[0].type);
-  EXPECT_EQ(kView1ID, event_list[0].view_id);
-  EXPECT_EQ(ON_FOCUS, event_list[1].type);
-  EXPECT_EQ(kView2ID, event_list[1].view_id);
+  ASSERT_EQ(2, static_cast<int>(event_list->vec.size()));
+  EXPECT_EQ(ON_BLUR, event_list->vec[0].type);
+  EXPECT_EQ(kView1ID, event_list->vec[0].view_id);
+  EXPECT_EQ(ON_FOCUS, event_list->vec[1].type);
+  EXPECT_EQ(kView2ID, event_list->vec[1].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[0].focus_change_reason);
+            event_list->vec[0].focus_change_reason);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[1].focus_change_reason);
+            event_list->vec[1].focus_change_reason);
 
-  event_list.clear();
+  event_list->vec.clear();
   GetFocusManager()->ClearFocus();
-  ASSERT_EQ(1, static_cast<int>(event_list.size()));
-  EXPECT_EQ(ON_BLUR, event_list[0].type);
-  EXPECT_EQ(kView2ID, event_list[0].view_id);
+  ASSERT_EQ(1, static_cast<int>(event_list->vec.size()));
+  EXPECT_EQ(ON_BLUR, event_list->vec[0].type);
+  EXPECT_EQ(kView2ID, event_list->vec[0].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[0].focus_change_reason);
+            event_list->vec[0].focus_change_reason);
 }
 
 TEST_F(FocusManagerTest, FocusChangeListener) {
@@ -148,6 +157,8 @@ TEST_F(FocusManagerTest, FocusChangeListener) {
   GetFocusManager()->ClearFocus();
   ASSERT_EQ(1, static_cast<int>(listener.focus_changes().size()));
   EXPECT_TRUE(listener.focus_changes()[0] == ViewPair(view2, null_view));
+
+  RemoveFocusChangeListener(&listener);
 }
 
 TEST_F(FocusManagerTest, WidgetFocusChangeListener) {
@@ -176,15 +187,17 @@ TEST_F(FocusManagerTest, WidgetFocusChangeListener) {
   gfx::NativeView native_view1 = widget1->GetNativeView();
   test::WidgetTest::SimulateNativeActivate(widget1.get());
   ASSERT_EQ(2u, widget_listener.focus_changes().size());
-  EXPECT_EQ(gfx::kNullNativeView, widget_listener.focus_changes()[0]);
+  EXPECT_EQ(gfx::NativeView(), widget_listener.focus_changes()[0]);
   EXPECT_EQ(native_view1, widget_listener.focus_changes()[1]);
 
   widget_listener.ClearFocusChanges();
   gfx::NativeView native_view2 = widget2->GetNativeView();
   test::WidgetTest::SimulateNativeActivate(widget2.get());
   ASSERT_EQ(2u, widget_listener.focus_changes().size());
-  EXPECT_EQ(gfx::kNullNativeView, widget_listener.focus_changes()[0]);
+  EXPECT_EQ(gfx::NativeView(), widget_listener.focus_changes()[0]);
   EXPECT_EQ(native_view2, widget_listener.focus_changes()[1]);
+
+  RemoveWidgetFocusChangeListener(&widget_listener);
 }
 
 TEST_F(FocusManagerTest, CallsNormalAcceleratorTarget) {
@@ -454,7 +467,7 @@ class FocusInAboutToRequestFocusFromTabTraversalView : public View {
   }
 
  private:
-  raw_ptr<views::View> view_to_focus_ = nullptr;
+  raw_ptr<views::View, AcrossTasksDanglingUntriaged> view_to_focus_ = nullptr;
 };
 }  // namespace
 
@@ -482,7 +495,7 @@ TEST_F(FocusManagerTest, FocusInAboutToRequestFocusFromTabTraversal) {
   EXPECT_TRUE(v1->HasFocus());
 }
 
-TEST_F(FocusManagerTest, RotatePaneFocus) {
+TEST_F(FocusManagerTest, RotateFocus) {
   views::AccessiblePaneView* pane1 = new AccessiblePaneView();
   GetContentsView()->AddChildView(pane1);
 
@@ -530,30 +543,6 @@ TEST_F(FocusManagerTest, RotatePaneFocus) {
   EXPECT_EQ(v4, focus_manager->GetFocusedView());
   focus_manager->AdvanceFocus(false);
   EXPECT_EQ(v3, focus_manager->GetFocusedView());
-
-  EXPECT_TRUE(focus_manager->RotatePaneFocus(Direction::kForward,
-                                             FocusCycleWrapping::kEnabled));
-  EXPECT_EQ(v1, focus_manager->GetFocusedView());
-
-  // Advance backwards.
-  EXPECT_TRUE(focus_manager->RotatePaneFocus(Direction::kBackward,
-                                             FocusCycleWrapping::kEnabled));
-  EXPECT_EQ(v3, focus_manager->GetFocusedView());
-
-  EXPECT_TRUE(focus_manager->RotatePaneFocus(Direction::kBackward,
-                                             FocusCycleWrapping::kEnabled));
-  EXPECT_EQ(v1, focus_manager->GetFocusedView());
-
-  // Advance without wrap. When it gets to the end of the list of
-  // panes, RotatePaneFocus should return false but the current
-  // focused view shouldn't change.
-  EXPECT_TRUE(focus_manager->RotatePaneFocus(Direction::kForward,
-                                             FocusCycleWrapping::kDisabled));
-  EXPECT_EQ(v3, focus_manager->GetFocusedView());
-
-  EXPECT_FALSE(focus_manager->RotatePaneFocus(Direction::kForward,
-                                              FocusCycleWrapping::kDisabled));
-  EXPECT_EQ(v3, focus_manager->GetFocusedView());
 }
 
 // Verifies the stored focus view tracks the focused view.
@@ -721,9 +710,9 @@ TEST_F(FocusManagerTest, SkipViewsInArrowKeyTraversal) {
 }
 
 TEST_F(FocusManagerTest, StoreFocusedView) {
-  std::vector<FocusTestEvent> event_list;
+  auto event_list = base::MakeRefCounted<FocusTestEventList>();
   const int kView1ID = 1;
-  SimpleTestView* view = new SimpleTestView(&event_list, kView1ID);
+  SimpleTestView* view = new SimpleTestView(event_list, kView1ID);
 
   // Add view to the view hierarchy and make it focusable.
   GetWidget()->GetRootView()->AddChildView(view);
@@ -734,36 +723,36 @@ TEST_F(FocusManagerTest, StoreFocusedView) {
   EXPECT_EQ(nullptr, GetFocusManager()->GetFocusedView());
   EXPECT_TRUE(GetFocusManager()->RestoreFocusedView());
   EXPECT_EQ(view, GetFocusManager()->GetStoredFocusView());
-  ASSERT_EQ(3, static_cast<int>(event_list.size()));
-  EXPECT_EQ(ON_FOCUS, event_list[0].type);
-  EXPECT_EQ(kView1ID, event_list[0].view_id);
+  ASSERT_EQ(3, static_cast<int>(event_list->vec.size()));
+  EXPECT_EQ(ON_FOCUS, event_list->vec[0].type);
+  EXPECT_EQ(kView1ID, event_list->vec[0].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[0].focus_change_reason);
-  EXPECT_EQ(ON_BLUR, event_list[1].type);
-  EXPECT_EQ(kView1ID, event_list[1].view_id);
+            event_list->vec[0].focus_change_reason);
+  EXPECT_EQ(ON_BLUR, event_list->vec[1].type);
+  EXPECT_EQ(kView1ID, event_list->vec[1].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[1].focus_change_reason);
-  EXPECT_EQ(ON_FOCUS, event_list[2].type);
-  EXPECT_EQ(kView1ID, event_list[2].view_id);
+            event_list->vec[1].focus_change_reason);
+  EXPECT_EQ(ON_FOCUS, event_list->vec[2].type);
+  EXPECT_EQ(kView1ID, event_list->vec[2].view_id);
   EXPECT_EQ(FocusChangeReason::kFocusRestore,
-            event_list[2].focus_change_reason);
+            event_list->vec[2].focus_change_reason);
 
   // Repeat with |true|.
-  event_list.clear();
+  event_list->vec.clear();
   GetFocusManager()->SetFocusedView(view);
   GetFocusManager()->StoreFocusedView(true);
   EXPECT_EQ(nullptr, GetFocusManager()->GetFocusedView());
   EXPECT_TRUE(GetFocusManager()->RestoreFocusedView());
   EXPECT_EQ(view, GetFocusManager()->GetStoredFocusView());
-  ASSERT_EQ(2, static_cast<int>(event_list.size()));
-  EXPECT_EQ(ON_BLUR, event_list[0].type);
-  EXPECT_EQ(kView1ID, event_list[0].view_id);
+  ASSERT_EQ(2, static_cast<int>(event_list->vec.size()));
+  EXPECT_EQ(ON_BLUR, event_list->vec[0].type);
+  EXPECT_EQ(kView1ID, event_list->vec[0].view_id);
   EXPECT_EQ(FocusChangeReason::kDirectFocusChange,
-            event_list[0].focus_change_reason);
-  EXPECT_EQ(ON_FOCUS, event_list[1].type);
-  EXPECT_EQ(kView1ID, event_list[1].view_id);
+            event_list->vec[0].focus_change_reason);
+  EXPECT_EQ(ON_FOCUS, event_list->vec[1].type);
+  EXPECT_EQ(kView1ID, event_list->vec[1].view_id);
   EXPECT_EQ(FocusChangeReason::kFocusRestore,
-            event_list[1].focus_change_reason);
+            event_list->vec[1].focus_change_reason);
 
   // Necessary for clean teardown.
   GetFocusManager()->ClearFocus();
@@ -1219,10 +1208,13 @@ class RedirectToParentFocusManagerTest : public FocusManagerTest {
   }
 
  protected:
-  raw_ptr<FocusManager> parent_focus_manager_ = nullptr;
-  raw_ptr<FocusManager> bubble_focus_manager_ = nullptr;
+  raw_ptr<FocusManager, AcrossTasksDanglingUntriaged> parent_focus_manager_ =
+      nullptr;
+  raw_ptr<FocusManager, AcrossTasksDanglingUntriaged> bubble_focus_manager_ =
+      nullptr;
 
-  raw_ptr<BubbleDialogDelegateView> bubble_ = nullptr;
+  raw_ptr<BubbleDialogDelegateView, AcrossTasksDanglingUntriaged> bubble_ =
+      nullptr;
 };
 
 // Test that when an accelerator is sent to a bubble that isn't registered,
