@@ -4,6 +4,8 @@
 
 package org.chromium.weblayer_private;
 
+import static org.chromium.cc.mojom.RootScrollOffsetUpdateFrequency.ALL_UPDATES;
+
 import android.Manifest.permission;
 import android.app.Activity;
 import android.content.pm.PackageManager;
@@ -31,13 +33,10 @@ import org.chromium.base.annotations.CalledByNative;
 import org.chromium.base.annotations.JNINamespace;
 import org.chromium.base.annotations.NativeMethods;
 import org.chromium.base.supplier.ObservableSupplier;
-import org.chromium.cc.input.BrowserControlsState;
-import org.chromium.components.autofill.AutofillActionModeCallback;
 import org.chromium.components.autofill.AutofillProvider;
+import org.chromium.components.autofill.AutofillSelectionMenuItemProvider;
 import org.chromium.components.browser_ui.display_cutout.DisplayCutoutController;
 import org.chromium.components.browser_ui.media.MediaSessionHelper;
-import org.chromium.components.browser_ui.util.BrowserControlsVisibilityDelegate;
-import org.chromium.components.browser_ui.util.ComposedBrowserControlsVisibilityDelegate;
 import org.chromium.components.browser_ui.widget.InsetObserverView;
 import org.chromium.components.embedder_support.contextmenu.ContextMenuParams;
 import org.chromium.components.embedder_support.util.UrlUtilities;
@@ -51,8 +50,11 @@ import org.chromium.components.webapps.AddToHomescreenCoordinator;
 import org.chromium.components.webapps.AppBannerManager;
 import org.chromium.content_public.browser.GestureListenerManager;
 import org.chromium.content_public.browser.GestureStateListener;
-import org.chromium.content_public.browser.GestureStateListenerWithScroll;
 import org.chromium.content_public.browser.LoadUrlParams;
+import org.chromium.content_public.browser.MessagePayload;
+import org.chromium.content_public.browser.MessagePayloadType;
+import org.chromium.content_public.browser.MessagePort;
+import org.chromium.content_public.browser.MessagePort.MessageCallback;
 import org.chromium.content_public.browser.NavigationHandle;
 import org.chromium.content_public.browser.SelectionClient;
 import org.chromium.content_public.browser.SelectionPopupController;
@@ -63,8 +65,8 @@ import org.chromium.content_public.browser.WebContentsObserver;
 import org.chromium.ui.base.ViewAndroidDelegate;
 import org.chromium.ui.base.WindowAndroid;
 import org.chromium.url.GURL;
-import org.chromium.weblayer_private.autofill_assistant.WebLayerAssistantTabChangeObserver;
 import org.chromium.weblayer_private.interfaces.APICallException;
+import org.chromium.weblayer_private.interfaces.ExceptionType;
 import org.chromium.weblayer_private.interfaces.IContextMenuParams;
 import org.chromium.weblayer_private.interfaces.IErrorPageCallbackClient;
 import org.chromium.weblayer_private.interfaces.IExternalIntentInIncognitoCallbackClient;
@@ -76,9 +78,9 @@ import org.chromium.weblayer_private.interfaces.IGoogleAccountsCallbackClient;
 import org.chromium.weblayer_private.interfaces.IMediaCaptureCallbackClient;
 import org.chromium.weblayer_private.interfaces.INavigationControllerClient;
 import org.chromium.weblayer_private.interfaces.IObjectWrapper;
+import org.chromium.weblayer_private.interfaces.IStringCallback;
 import org.chromium.weblayer_private.interfaces.ITab;
 import org.chromium.weblayer_private.interfaces.ITabClient;
-import org.chromium.weblayer_private.interfaces.IWebMessageCallbackClient;
 import org.chromium.weblayer_private.interfaces.ObjectWrapper;
 import org.chromium.weblayer_private.interfaces.ScrollNotificationType;
 import org.chromium.weblayer_private.interfaces.StrictModeWorkaround;
@@ -88,7 +90,6 @@ import org.chromium.weblayer_private.media.MediaStreamManager;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
@@ -112,6 +113,7 @@ public final class TabImpl extends ITab.Stub {
     private TabViewAndroidDelegate mViewAndroidDelegate;
     private GoogleAccountsCallbackProxy mGoogleAccountsCallbackProxy;
     private ExternalIntentInIncognitoCallbackProxy mExternalIntentInIncognitoCallbackProxy;
+    private MessagePort[] mChannel;
     // BrowserImpl this TabImpl is in.
     @NonNull
     private BrowserImpl mBrowser;
@@ -125,18 +127,6 @@ public final class TabImpl extends ITab.Stub {
     private ITabClient mClient;
     private final int mId;
 
-    // A list of browser control visibility constraints, indexed by ImplControlsVisibilityReason.
-    private List<BrowserControlsVisibilityDelegate> mBrowserControlsDelegates;
-    // Computes a net browser control visibility constraint from constituent constraints.
-    private ComposedBrowserControlsVisibilityDelegate mComposedBrowserControlsVisibility;
-    // Which BrowserControlsVisibilityDelegate is currently controlling the visibility. The active
-    // delegate changes from mComposedBrowserControlsVisibility to the delegate for visibility
-    // reason RENDERER_UNAVAILABLE if onlyExpandControlsAtPageTop is enabled, in which case we don't
-    // want to ever force the controls to be visible unless the renderer isn't responsive.
-    private BrowserControlsVisibilityDelegate mActiveBrowserControlsVisibilityDelegate;
-    // Invoked when the computed visibility constraint changes.
-    private Callback<Integer> mConstraintsUpdatedCallback;
-
     private IFindInPageCallbackClient mFindInPageCallbackClient;
     private FindInPageBridge mFindInPageBridge;
     private FindResultBar mFindResultBar;
@@ -149,17 +139,20 @@ public final class TabImpl extends ITab.Stub {
     private DisplayCutoutController mDisplayCutoutController;
 
     private boolean mPostContainerViewInitDone;
-    private ActionModeCallback mActionModeCallback;
-
-    private WebLayerAccessibilityUtil.Observer mAccessibilityObserver;
-
-    private final WebLayerAssistantTabChangeObserver mWebLayerAssistantTabChangeObserver =
-            new WebLayerAssistantTabChangeObserver();
+    private WebLayerActionModeCallback mActionModeCallback;
 
     private Set<FaviconCallbackProxy> mFaviconCallbackProxies = new HashSet<>();
 
     // Only non-null if scroll offsets have been requested.
-    private @Nullable GestureStateListenerWithScroll mGestureStateListenerWithScroll;
+    private @Nullable GestureStateListener mGestureStateListener;
+
+    private HeaderVerificationStatus mHeaderVerification;
+
+    enum HeaderVerificationStatus {
+        PENDING,
+        NOT_VALIDATED,
+        VALIDATED,
+    }
 
     private static class InternalAccessDelegateImpl
             implements ViewEventSink.InternalAccessDelegate {
@@ -198,23 +191,6 @@ public final class TabImpl extends ITab.Stub {
             mIgnoreRenderer = ignoreRenderer;
         }
 
-        @Override
-        public void onTopControlsChanged(
-                int topControlsOffsetY, int topContentOffsetY, int topControlsMinHeightOffsetY) {
-            BrowserViewController viewController = getViewController();
-            if (viewController != null && !mIgnoreRenderer) {
-                viewController.onTopControlsChanged(topControlsOffsetY, topContentOffsetY);
-            }
-        }
-        @Override
-        public void onBottomControlsChanged(
-                int bottomControlsOffsetY, int bottomControlsMinHeightOffsetY) {
-            BrowserViewController viewController = getViewController();
-            if (viewController != null && !mIgnoreRenderer) {
-                viewController.onBottomControlsChanged(bottomControlsOffsetY);
-            }
-        }
-
         @Override
         public void onBackgroundColorChanged(int color) {
             try {
@@ -239,6 +215,22 @@ public final class TabImpl extends ITab.Stub {
         }
     }
 
+    class NativeStringCallback implements Callback<String> {
+        private IStringCallback mCallback;
+
+        NativeStringCallback(IStringCallback callback) {
+            mCallback = callback;
+        }
+
+        @Override
+        public void onResult(String result) {
+            try {
+                mCallback.onResult(result);
+            } catch (RemoteException e) {
+            }
+        }
+    }
+
     public static TabImpl fromWebContents(WebContents webContents) {
         if (webContents == null || webContents.isDestroyed()) return null;
         return TabImplJni.get().fromWebContents(webContents);
@@ -282,11 +274,6 @@ public final class TabImpl extends ITab.Stub {
                 }
             }
 
-            @Override
-            public void didStartNavigationNoop(NavigationHandle navigationHandle) {
-                if (!navigationHandle.isInPrimaryMainFrame()) return;
-            }
-
             @Override
             public void viewportFitChanged(@WebContentsObserver.ViewportFitType int value) {
                 ensureDisplayCutoutController();
@@ -295,20 +282,9 @@ public final class TabImpl extends ITab.Stub {
         };
         mWebContents.addObserver(mWebContentsObserver);
 
-        mMediaStreamManager = new MediaStreamManager(this);
+        mHeaderVerification = HeaderVerificationStatus.PENDING;
 
-        mBrowserControlsDelegates = new ArrayList<BrowserControlsVisibilityDelegate>();
-        mComposedBrowserControlsVisibility = new ComposedBrowserControlsVisibilityDelegate();
-        for (int i = 0; i < ImplControlsVisibilityReason.REASON_COUNT; ++i) {
-            BrowserControlsVisibilityDelegate delegate =
-                    new BrowserControlsVisibilityDelegate(BrowserControlsState.BOTH);
-            mBrowserControlsDelegates.add(delegate);
-            mComposedBrowserControlsVisibility.addDelegate(delegate);
-        }
-        mConstraintsUpdatedCallback =
-                (constraint) -> onBrowserControlsConstraintUpdated(constraint);
-        mActiveBrowserControlsVisibilityDelegate = mComposedBrowserControlsVisibility;
-        mActiveBrowserControlsVisibilityDelegate.addObserver(mConstraintsUpdatedCallback);
+        mMediaStreamManager = new MediaStreamManager(this);
 
         mInterceptNavigationDelegateClient = new InterceptNavigationDelegateClientImpl(this);
         mInterceptNavigationDelegate =
@@ -317,12 +293,6 @@ public final class TabImpl extends ITab.Stub {
         sTabMap.put(mId, this);
 
         mInfoBarContainer = new InfoBarContainer(this);
-        mAccessibilityObserver = (boolean enabled) -> {
-            setBrowserControlsVisibilityConstraint(ImplControlsVisibilityReason.ACCESSIBILITY,
-                    enabled ? BrowserControlsState.SHOWN : BrowserControlsState.BOTH);
-        };
-        // addObserver() calls to observer when added.
-        WebLayerAccessibilityUtil.get().addObserver(mAccessibilityObserver);
 
         mMediaSessionHelper = new MediaSessionHelper(
                 mWebContents, MediaSessionManager.createMediaSessionHelperDelegate(this));
@@ -339,18 +309,23 @@ public final class TabImpl extends ITab.Stub {
                             throw new APICallException(e);
                         }
                     }
-                });
+                }, ALL_UPDATES);
     }
 
     private void doInitAfterSettingContainerView() {
         if (mPostContainerViewInitDone) return;
 
-        mPostContainerViewInitDone = true;
-        SelectionPopupController controller =
-                SelectionPopupController.fromWebContents(mWebContents);
-        mActionModeCallback = new ActionModeCallback(mWebContents);
-        controller.setActionModeCallback(mActionModeCallback);
-        controller.setSelectionClient(SelectionClient.createSmartSelectionClient(mWebContents));
+        if (mBrowser.getBrowserFragment().getViewAndroidDelegateContainerView() != null) {
+            mPostContainerViewInitDone = true;
+
+            SelectionPopupController selectionPopupController =
+                    SelectionPopupController.fromWebContents(mWebContents);
+            mActionModeCallback = new WebLayerActionModeCallback(mWebContents);
+            mActionModeCallback.setTabClient(mClient);
+            selectionPopupController.setActionModeCallback(mActionModeCallback);
+            selectionPopupController.setSelectionClient(
+                    SelectionClient.createSmartSelectionClient(mWebContents));
+        }
     }
 
     public ProfileImpl getProfile() {
@@ -361,6 +336,10 @@ public final class TabImpl extends ITab.Stub {
         return mClient;
     }
 
+    public void setHeaderVerification(HeaderVerificationStatus headerVerification) {
+        mHeaderVerification = headerVerification;
+    }
+
     /**
      * Sets the BrowserImpl this TabImpl is contained in.
      */
@@ -373,48 +352,49 @@ public final class TabImpl extends ITab.Stub {
     }
 
     public void updateFromBrowser() {
-        mWebContents.setTopLevelNativeWindow(mBrowser.getWindowAndroid());
-        mViewAndroidDelegate.setContainerView(mBrowser.getViewAndroidDelegateContainerView());
+        mViewAndroidDelegate.setContainerView(
+                mBrowser.getBrowserFragment().getViewAndroidDelegateContainerView());
         doInitAfterSettingContainerView();
         updateViewAttachedStateFromBrowser();
 
-        boolean attached = (mBrowser.getContext() != null);
+        boolean attached = mBrowser.getBrowserFragment().isAttached();
         mInterceptNavigationDelegateClient.onActivityAttachmentChanged(attached);
 
         if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
             SelectionPopupController selectionController =
-                    SelectionPopupController.fromWebContents(mWebContents);
-            if (mBrowser.getContext() == null) {
+                    SelectionPopupController.fromWebContentsNoCreate(mWebContents);
+            if (!attached) {
                 // The Context and ViewContainer in which Autofill was previously operating have
                 // gone away, so tear down |mAutofillProvider|.
                 if (mAutofillProvider != null) {
                     mAutofillProvider.destroy();
                     mAutofillProvider = null;
                 }
-                selectionController.setNonSelectionActionModeCallback(null);
+                if (selectionController != null) {
+                    selectionController.setNonSelectionAdditionalMenuItemProvider(null);
+                }
             } else {
                 if (mAutofillProvider == null) {
                     // Set up |mAutofillProvider| to operate in the new Context. It's safe to assume
                     // the context won't change unless it is first nulled out, since the fragment
                     // must be detached before it can be reattached to a new Context.
                     mAutofillProvider = new AutofillProvider(mBrowser.getContext(),
-                            mBrowser.getViewAndroidDelegateContainerView(), mWebContents,
-                            "WebLayer");
+                            mBrowser.getBrowserFragment().getViewAndroidDelegateContainerView(),
+                            mWebContents, "WebLayer");
                     TabImplJni.get().initializeAutofillIfNecessary(mNativeTab);
                 }
                 mAutofillProvider.onContainerViewChanged(
-                        mBrowser.getViewAndroidDelegateContainerView());
+                        mBrowser.getBrowserFragment().getViewAndroidDelegateContainerView());
                 mAutofillProvider.setWebContents(mWebContents);
-
-                selectionController.setNonSelectionActionModeCallback(
-                        new AutofillActionModeCallback(mBrowser.getContext(), mAutofillProvider));
+                if (selectionController != null) {
+                    selectionController.setNonSelectionAdditionalMenuItemProvider(
+                            new AutofillSelectionMenuItemProvider(
+                                    mBrowser.getContext(), mAutofillProvider));
+                }
             }
         }
-
-        mWebLayerAssistantTabChangeObserver.onBrowserAttachmentChanged(mWebContents);
     }
 
-    @VisibleForTesting
     public AutofillProvider getAutofillProviderForTesting() {
         // The test needs to make sure the |mAutofillProvider| is not null.
         return mAutofillProvider;
@@ -456,14 +436,18 @@ public final class TabImpl extends ITab.Stub {
         return mId;
     }
 
+    @Override
+    public String getUri() {
+        StrictModeWorkaround.apply();
+        return mWebContents.getVisibleUrl().getSpec();
+    }
+
     /**
      * Called when this TabImpl is attached to the BrowserViewController.
      */
-    public void onAttachedToViewController(
-            long topControlsContainerViewHandle, long bottomControlsContainerViewHandle) {
-        // attachToFragment() must be called before activate().
-        TabImplJni.get().setBrowserControlsContainerViews(
-                mNativeTab, topControlsContainerViewHandle, bottomControlsContainerViewHandle);
+    public void onAttachedToViewController() {
+        mWebContents.setTopLevelNativeWindow(mBrowser.getBrowserFragment().getWindowAndroid());
+        doInitAfterSettingContainerView();
         mInfoBarContainer.onTabAttachedToViewController();
         updateWebContentsVisibility();
         updateDisplayCutoutController();
@@ -473,6 +457,7 @@ public final class TabImpl extends ITab.Stub {
      * Called when this TabImpl is detached from the BrowserViewController.
      */
     public void onDetachedFromViewController() {
+        mWebContents.setTopLevelNativeWindow(null);
         if (mAutofillProvider != null) {
             mAutofillProvider.hidePopup();
         }
@@ -488,15 +473,13 @@ public final class TabImpl extends ITab.Stub {
         if (mInfoBarContainer != null) {
             mInfoBarContainer.onTabDetachedFromViewController();
         }
-
-        TabImplJni.get().setBrowserControlsContainerViews(mNativeTab, 0, 0);
     }
 
     /**
      * Returns whether this Tab is visible.
      */
     public boolean isVisible() {
-        return isActiveTab() && mBrowser.isActiveTabVisible();
+        return isActiveTab() && mBrowser.getBrowserFragment().isVisible();
     }
 
     @CalledByNative
@@ -515,6 +498,7 @@ public final class TabImpl extends ITab.Stub {
     }
 
     private void updateWebContentsVisibility() {
+        if (SelectionPopupController.fromWebContentsNoCreate(mWebContents) == null) return;
         boolean visibleNow = isVisible();
         boolean webContentsVisible = mWebContents.getVisibility() == Visibility.VISIBLE;
         if (visibleNow) {
@@ -527,7 +511,8 @@ public final class TabImpl extends ITab.Stub {
     private void updateDisplayCutoutController() {
         if (mDisplayCutoutController == null) return;
 
-        mDisplayCutoutController.onActivityAttachmentChanged(mBrowser.getWindowAndroid());
+        mDisplayCutoutController.onActivityAttachmentChanged(
+                mBrowser.getBrowserFragment().getWindowAndroid());
         mDisplayCutoutController.maybeUpdateLayout();
     }
 
@@ -561,7 +546,6 @@ public final class TabImpl extends ITab.Stub {
         return mNativeTab;
     }
 
-    @VisibleForTesting
     public InfoBarContainer getInfoBarContainerForTesting() {
         return mInfoBarContainer;
     }
@@ -580,7 +564,6 @@ public final class TabImpl extends ITab.Stub {
         StrictModeWorkaround.apply();
         mClient = client;
         mTabCallbackProxy = new TabCallbackProxy(mNativeTab, client);
-        mActionModeCallback.setTabClient(mClient);
     }
 
     @Override
@@ -650,8 +633,8 @@ public final class TabImpl extends ITab.Stub {
     public void setScrollOffsetsEnabled(boolean enabled) {
         StrictModeWorkaround.apply();
         if (enabled) {
-            if (mGestureStateListenerWithScroll == null) {
-                mGestureStateListenerWithScroll = new GestureStateListenerWithScroll() {
+            if (mGestureStateListener == null) {
+                mGestureStateListener = new GestureStateListener() {
                     @Override
                     public void onScrollOffsetOrExtentChanged(
                             int scrollOffsetY, int scrollExtentY) {
@@ -663,12 +646,12 @@ public final class TabImpl extends ITab.Stub {
                     }
                 };
                 GestureListenerManager.fromWebContents(mWebContents)
-                        .addListener(mGestureStateListenerWithScroll);
+                        .addListener(mGestureStateListener, ALL_UPDATES);
             }
-        } else if (mGestureStateListenerWithScroll != null) {
+        } else if (mGestureStateListener != null) {
             GestureListenerManager.fromWebContents(mWebContents)
-                    .removeListener(mGestureStateListenerWithScroll);
-            mGestureStateListenerWithScroll = null;
+                    .removeListener(mGestureStateListener);
+            mGestureStateListener = null;
         }
     }
 
@@ -696,7 +679,7 @@ public final class TabImpl extends ITab.Stub {
         NativeContextMenuParamsHolder nativeContextMenuParamsHolder =
                 (NativeContextMenuParamsHolder) contextMenuParams;
 
-        WindowAndroid window = getBrowser().getWindowAndroid();
+        WindowAndroid window = getBrowser().getBrowserFragment().getWindowAndroid();
         if (window.hasPermission(permission.WRITE_EXTERNAL_STORAGE)) {
             continueDownload(nativeContextMenuParamsHolder);
             return;
@@ -717,6 +700,7 @@ public final class TabImpl extends ITab.Stub {
 
     @Override
     public void addToHomescreen() {
+        StrictModeWorkaround.apply();
         // TODO(estade): should it be verified that |this| is the active tab?
 
         // This is used for UMA, and is only meaningful for Chrome. TODO(estade): remove.
@@ -724,7 +708,8 @@ public final class TabImpl extends ITab.Stub {
         menuItemData.putInt(AppBannerManager.MENU_TITLE_KEY, 0);
         // TODO(estade): simplify these parameters.
         AddToHomescreenCoordinator.showForAppMenu(mBrowser.getContext(),
-                mBrowser.getWindowAndroid(), mBrowser.getWindowAndroid().getModalDialogManager(),
+                mBrowser.getBrowserFragment().getWindowAndroid(),
+                mBrowser.getBrowserFragment().getWindowAndroid().getModalDialogManager(),
                 mWebContents, menuItemData);
     }
 
@@ -753,35 +738,32 @@ public final class TabImpl extends ITab.Stub {
     }
 
     @Override
-    public void executeScript(String script, boolean useSeparateIsolate, IObjectWrapper callback) {
+    public void executeScript(String script, boolean useSeparateIsolate, IStringCallback callback) {
         StrictModeWorkaround.apply();
+        if (mHeaderVerification == HeaderVerificationStatus.VALIDATED) {
+            TabImplJni.get().executeScript(
+                    mNativeTab, script, useSeparateIsolate, new NativeStringCallback(callback));
+            return;
+        }
 
         WebLayerOriginVerificationScheduler originVerifier =
                 WebLayerOriginVerificationScheduler.getInstance();
         String url = mWebContents.getVisibleUrl().getSpec();
-        originVerifier.verify(url, mProfile, (verified) -> {
+        originVerifier.verify(url, (verified) -> {
             // Make sure the page hasn't changed since we started verification.
             if (!url.equals(mWebContents.getVisibleUrl().getSpec())) {
                 return;
             }
 
-            ValueCallback<String> unwrappedCallback =
-                    (ValueCallback<String>) ObjectWrapper.unwrap(callback, ValueCallback.class);
-            assert unwrappedCallback != null;
-
             if (!verified) {
-                // TODO(swestphal): Propagate exception to calling api.
-                unwrappedCallback.onReceiveValue(null);
-                return;
-            }
-
-            Callback<String> nativeCallback = new Callback<String>() {
-                @Override
-                public void onResult(String result) {
-                    unwrappedCallback.onReceiveValue(result);
+                try {
+                    callback.onException(ExceptionType.RESTRICTED_API,
+                            "Application does not have permissions to modify " + url);
+                } catch (RemoteException e) {
                 }
-            };
-            TabImplJni.get().executeScript(mNativeTab, script, useSeparateIsolate, nativeCallback);
+            }
+            TabImplJni.get().executeScript(
+                    mNativeTab, script, useSeparateIsolate, new NativeStringCallback(callback));
         });
     }
 
@@ -800,21 +782,13 @@ public final class TabImpl extends ITab.Stub {
         BrowserViewController controller = getViewController();
         if (controller == null) return false;
 
-        // Refuse to start a find session when the browser controls are forced hidden.
-        if (mActiveBrowserControlsVisibilityDelegate.get() == BrowserControlsState.HIDDEN) {
-            return false;
-        }
-
-        setBrowserControlsVisibilityConstraint(
-                ImplControlsVisibilityReason.FIND_IN_PAGE, BrowserControlsState.SHOWN);
-
         mFindInPageCallbackClient = client;
         assert mFindInPageBridge == null;
         mFindInPageBridge = new FindInPageBridge(mWebContents);
         assert mFindResultBar == null;
         mFindResultBar =
                 new FindResultBar(mBrowser.getContext(), controller.getWebContentsOverlayView(),
-                        mBrowser.getWindowAndroid(), mFindInPageBridge);
+                        mBrowser.getBrowserFragment().getWindowAndroid(), mFindInPageBridge);
         return true;
     }
 
@@ -839,9 +813,6 @@ public final class TabImpl extends ITab.Stub {
         mFindInPageBridge.destroy();
         mFindInPageBridge = null;
 
-        setBrowserControlsVisibilityConstraint(
-                ImplControlsVisibilityReason.FIND_IN_PAGE, BrowserControlsState.BOTH);
-
         try {
             if (mFindInPageCallbackClient != null) mFindInPageCallbackClient.onFindEnded();
             mFindInPageCallbackClient = null;
@@ -867,7 +838,8 @@ public final class TabImpl extends ITab.Stub {
             return true;
         }
 
-        SelectionPopupController popup = SelectionPopupController.fromWebContents(mWebContents);
+        SelectionPopupController popup =
+                SelectionPopupController.fromWebContentsNoCreate(mWebContents);
         if (popup != null && popup.isSelectActionBarShowing()) {
             popup.clearSelection();
             return true;
@@ -985,11 +957,13 @@ public final class TabImpl extends ITab.Stub {
 
     @Override
     public void setMediaCaptureCallbackClient(IMediaCaptureCallbackClient client) {
+        StrictModeWorkaround.apply();
         mMediaStreamManager.setClient(client);
     }
 
     @Override
     public void stopMediaCapturing() {
+        StrictModeWorkaround.apply();
         mMediaStreamManager.stopStreaming();
     }
 
@@ -999,30 +973,36 @@ public final class TabImpl extends ITab.Stub {
         getBrowser().destroyTab(this);
     }
 
-    @Override
-    public void registerWebMessageCallback(
-            String jsObjectName, List<String> allowedOrigins, IWebMessageCallbackClient client) {
-        if (jsObjectName.isEmpty()) {
-            throw new IllegalArgumentException("JS object name must not be empty");
-        }
-        if (allowedOrigins.isEmpty()) {
-            throw new IllegalArgumentException("At least one origin must be specified");
-        }
-        for (String origin : allowedOrigins) {
-            if (TextUtils.isEmpty(origin)) {
-                throw new IllegalArgumentException("Origin must not be non-empty");
-            }
-        }
-        String registerError = TabImplJni.get().registerWebMessageCallback(mNativeTab, jsObjectName,
-                allowedOrigins.toArray(new String[allowedOrigins.size()]), client);
-        if (!TextUtils.isEmpty(registerError)) {
-            throw new IllegalArgumentException(registerError);
-        }
+    private String getAppOrigin() {
+        // TODO(rayankans): Consider exposing the embedder app's fingerprints as well.
+        return "app://" + mBrowser.getContext().getPackageName();
     }
 
     @Override
-    public void unregisterWebMessageCallback(String jsObjectName) {
-        TabImplJni.get().unregisterWebMessageCallback(mNativeTab, jsObjectName);
+    public void postMessage(String message, String targetOrigin) {
+        StrictModeWorkaround.apply();
+
+        if (mChannel == null || mChannel[0].isClosed() || mChannel[0].isTransferred()
+                || mChannel[1].isClosed() || mChannel[1].isTransferred()) {
+            mChannel = mWebContents.createMessageChannel();
+            mChannel[0].setMessageCallback(new MessageCallback() {
+                @Override
+                public void onMessage(MessagePayload messagePayload, MessagePort[] sentPorts) {
+                    try {
+                        if (messagePayload.getType() == MessagePayloadType.ARRAY_BUFFER) {
+                            // TODO(rayankans): Consider supporting passing array buffers.
+                            return;
+                        }
+                        mClient.onPostMessage(messagePayload.getAsString(),
+                                mWebContents.getVisibleUrl().getOrigin().getSpec());
+                    } catch (RemoteException e) {
+                    }
+                }
+            }, null);
+        }
+
+        mWebContents.postMessageToMainFrame(new MessagePayload(message), getAppOrigin(),
+                targetOrigin, new MessagePort[] {mChannel[1]});
     }
 
     public void destroy() {
@@ -1031,8 +1011,6 @@ public final class TabImpl extends ITab.Stub {
 
         TabImplJni.get().removeTabFromBrowserBeforeDestroying(mNativeTab);
 
-        mWebLayerAssistantTabChangeObserver.onTabDestroyed(mWebContents);
-
         // Notify the client that this instance is being destroyed to prevent it from calling
         // back into this object if the embedder mistakenly tries to do so.
         try {
@@ -1101,47 +1079,12 @@ public final class TabImpl extends ITab.Stub {
 
         sTabMap.remove(mId);
 
-        // ObservableSupplierImpl.addObserver() posts a task to notify the observer, ensure the
-        // callback isn't run after destroy() is called (otherwise we'll get crashes as the native
-        // tab has been deleted).
-        mActiveBrowserControlsVisibilityDelegate.removeObserver(mConstraintsUpdatedCallback);
         hideFindInPageUiAndNotifyClient();
         mFindInPageCallbackClient = null;
         mNavigationController = null;
         mWebContents.removeObserver(mWebContentsObserver);
         TabImplJni.get().deleteTab(mNativeTab);
         mNativeTab = 0;
-
-        WebLayerAccessibilityUtil.get().removeObserver(mAccessibilityObserver);
-    }
-
-    @CalledByNative
-    private boolean doBrowserControlsShrinkRendererSize() {
-        BrowserViewController viewController = getViewController();
-        return viewController != null && viewController.doBrowserControlsShrinkRendererSize();
-    }
-
-    @CalledByNative
-    public void setBrowserControlsVisibilityConstraint(
-            @ImplControlsVisibilityReason int reason, @BrowserControlsState int constraint) {
-        mBrowserControlsDelegates.get(reason).set(constraint);
-    }
-
-    @BrowserControlsState
-    /* package */ int getBrowserControlsVisibilityConstraint(
-            @ImplControlsVisibilityReason int reason) {
-        return mBrowserControlsDelegates.get(reason).get();
-    }
-
-    public void setOnlyExpandTopControlsAtPageTop(boolean onlyExpandControlsAtPageTop) {
-        BrowserControlsVisibilityDelegate activeDelegate = onlyExpandControlsAtPageTop
-                ? mBrowserControlsDelegates.get(ImplControlsVisibilityReason.RENDERER_UNAVAILABLE)
-                : mComposedBrowserControlsVisibility;
-        if (activeDelegate == mActiveBrowserControlsVisibilityDelegate) return;
-
-        mActiveBrowserControlsVisibilityDelegate.removeObserver(mConstraintsUpdatedCallback);
-        mActiveBrowserControlsVisibilityDelegate = activeDelegate;
-        mActiveBrowserControlsVisibilityDelegate.addObserver(mConstraintsUpdatedCallback);
     }
 
     @CalledByNative
@@ -1209,48 +1152,12 @@ public final class TabImpl extends ITab.Stub {
                 new NativeContextMenuParamsHolder(nativeContextMenuParams));
     }
 
-    @VisibleForTesting
-    public boolean canBrowserControlsScrollForTesting() {
-        return mActiveBrowserControlsVisibilityDelegate.get() == BrowserControlsState.BOTH;
-    }
-
     @VisibleForTesting
     public boolean didShowFullscreenToast() {
         return mFullscreenCallbackProxy != null
                 && mFullscreenCallbackProxy.didShowFullscreenToast();
     }
 
-    private void onBrowserControlsConstraintUpdated(int constraint) {
-        // If something has overridden the FIP's SHOWN constraint, cancel FIP. This causes FIP to
-        // dismiss when entering fullscreen.
-        if (constraint != BrowserControlsState.SHOWN) {
-            hideFindInPageUiAndNotifyClient();
-        }
-
-        // Don't animate when hiding the controls unless an animation was requested by
-        // BrowserControlsContainerView.
-        BrowserViewController viewController = getViewController();
-        boolean animate = constraint != BrowserControlsState.HIDDEN
-                || (viewController != null
-                        && viewController.shouldAnimateBrowserControlsHeightChanges());
-
-        // If the renderer is not controlling the offsets (possibly hung or crashed). Then this
-        // needs to force the controls to show (because notification from the renderer will not
-        // happen). For js dialogs, the renderer's update will come when the dialog is hidden, and
-        // since that animates from 0 height, it causes a flicker since the override is already set
-        // to fully show. Thus, disable animation.
-        if (constraint == BrowserControlsState.SHOWN && isActiveTab()
-                && !TabImplJni.get().isRendererControllingBrowserControlsOffsets(mNativeTab)) {
-            mViewAndroidDelegate.setIgnoreRendererUpdates(true);
-            if (viewController != null) viewController.showControls();
-            animate = false;
-        } else {
-            mViewAndroidDelegate.setIgnoreRendererUpdates(false);
-        }
-
-        TabImplJni.get().updateBrowserControlsConstraint(mNativeTab, constraint, animate);
-    }
-
     private void ensureDisplayCutoutController() {
         if (mDisplayCutoutController != null) return;
 
@@ -1258,7 +1165,7 @@ public final class TabImpl extends ITab.Stub {
                 new DisplayCutoutController(new DisplayCutoutController.Delegate() {
                     @Override
                     public Activity getAttachedActivity() {
-                        WindowAndroid window = mBrowser.getWindowAndroid();
+                        WindowAndroid window = mBrowser.getBrowserFragment().getWindowAndroid();
                         return window == null ? null : window.getActivity().get();
                     }
 
@@ -1269,7 +1176,8 @@ public final class TabImpl extends ITab.Stub {
 
                     @Override
                     public InsetObserverView getInsetObserverView() {
-                        BrowserViewController controller = mBrowser.getPossiblyNullViewController();
+                        BrowserViewController controller =
+                                mBrowser.getBrowserFragment().getPossiblyNullViewController();
                         return controller != null ? controller.getInsetObserverView() : null;
                     }
 
@@ -1302,16 +1210,15 @@ public final class TabImpl extends ITab.Stub {
         // During rotation it's possible for this to be called before BrowserViewController has been
         // updated. Verify BrowserViewController reflects this is the active tab before returning
         // it.
-        BrowserViewController viewController = mBrowser.getPossiblyNullViewController();
+        BrowserViewController viewController =
+                mBrowser.getBrowserFragment().getPossiblyNullViewController();
         return viewController != null && viewController.getTab() == this ? viewController : null;
     }
 
-    @VisibleForTesting
     public boolean canInfoBarContainerScrollForTesting() {
         return mInfoBarContainer.getContainerViewForTesting().isAllowedToAutoHide();
     }
 
-    @VisibleForTesting
     public String getTranslateInfoBarTargetLanguageForTesting() {
         if (!mInfoBarContainer.hasInfoBars()) return null;
 
@@ -1336,23 +1243,14 @@ public final class TabImpl extends ITab.Stub {
         void deleteTab(long tab);
         void setJavaImpl(long nativeTabImpl, TabImpl impl);
         void initializeAutofillIfNecessary(long nativeTabImpl);
-        void setBrowserControlsContainerViews(long nativeTabImpl,
-                long nativeTopBrowserControlsContainerView,
-                long nativeBottomBrowserControlsContainerView);
         WebContents getWebContents(long nativeTabImpl);
         void executeScript(long nativeTabImpl, String script, boolean useSeparateIsolate,
                 Callback<String> callback);
-        void updateBrowserControlsConstraint(
-                long nativeTabImpl, int newConstraint, boolean animate);
         String getGuid(long nativeTabImpl);
         void captureScreenShot(long nativeTabImpl, float scale,
                 ValueCallback<Pair<Bitmap, Integer>> valueCallback);
         boolean setData(long nativeTabImpl, String[] data);
         String[] getData(long nativeTabImpl);
-        boolean isRendererControllingBrowserControlsOffsets(long nativeTabImpl);
-        String registerWebMessageCallback(long nativeTabImpl, String jsObjectName,
-                String[] allowedOrigins, IWebMessageCallbackClient client);
-        void unregisterWebMessageCallback(long nativeTabImpl, String jsObjectName);
         boolean canTranslate(long nativeTabImpl);
         void showTranslateUi(long nativeTabImpl);
         void setTranslateTargetLanguage(long nativeTabImpl, String targetLanguage);
