@@ -1,10 +1,10 @@
-// Copyright 2022 The Chromium Authors. All rights reserved.
+// Copyright 2022 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 package org.chromium.weblayer_private;
 
-import org.chromium.components.digital_asset_links.VerificationResultStore;
+import org.chromium.components.content_relationship_verification.VerificationResultStore;
 
 import java.util.Collections;
 import java.util.HashSet;
