@@ -15,6 +15,7 @@
 #include "favicon_service_factory_qt.h"
 #include "find_text_helper.h"
 #include "media_capture_devices_dispatcher.h"
+#include "pdf_util_qt.h"
 #include "profile_adapter.h"
 #include "profile_qt.h"
 #include "qwebengineloadinginfo.h"
@@ -39,7 +40,6 @@
 #include "content/browser/renderer_host/render_view_host_impl.h"
 #include "content/browser/renderer_host/text_input_manager.h"
 #include "content/browser/web_contents/web_contents_impl.h"
-#include "content/public/browser/child_process_security_policy.h"
 #include "content/public/browser/devtools_agent_host.h"
 #include "content/public/browser/download_manager.h"
 #include "content/public/browser/download_request_utils.h"
@@ -60,6 +60,7 @@
 #include "ui/base/clipboard/clipboard_constants.h"
 #include "ui/base/clipboard/custom_data_helper.h"
 #include "ui/gfx/font_render_params.h"
+#include "ui/native_theme/native_theme.h"
 #include "qtwebengine/browser/qtwebenginepage.mojom.h"
 
 #include <QtCore/QVariant>
@@ -80,8 +81,6 @@
 #endif
 
 #if QT_CONFIG(webengine_printing_and_pdf)
-#include "components/pdf/browser/pdf_web_contents_helper.h"
-#include "printing/pdf_web_contents_helper_client_qt.h"
 #include "printing/print_view_manager_qt.h"
 #endif
 
@@ -152,7 +151,7 @@ static QVariant fromJSValue(const base::Value *result)
         }
         break;
     }
-    case base::Value::Type::DICTIONARY:
+    case base::Value::Type::DICT:
     {
         if (const auto dict = result->GetIfDict()) {
             QVariantMap map;
@@ -210,10 +209,28 @@ static std::unique_ptr<content::WebContents> createBlankWebContents(WebContentsA
     return webContents;
 }
 
+static int navigationListSize(content::NavigationController &controller) {
+    // If we're currently on the initial NavigationEntry, no navigation has
+    // committed, so the initial NavigationEntry should not be part of the
+    // "Navigation List", and we should return 0 as the navigation list size.
+    if (controller.GetLastCommittedEntry()->IsInitialEntry())
+        return 0;
+    return controller.GetEntryCount();
+}
+
+static int navigationListCurrentIndex(content::NavigationController &controller) {
+    // If we're currently on the initial NavigationEntry, no navigation has
+    // committed, so the initial NavigationEntry should not be part of the
+    // "Navigation List", and we should return -1 as the current index.
+    if (controller.GetLastCommittedEntry()->IsInitialEntry())
+        return -1;
+    return controller.GetCurrentEntryIndex();
+}
+
 static void serializeNavigationHistory(content::NavigationController &controller, QDataStream &output)
 {
-    const int currentIndex = controller.GetCurrentEntryIndex();
-    const int count = controller.GetEntryCount();
+    const int currentIndex = navigationListCurrentIndex(controller);
+    const int count = navigationListSize(controller);
     const int pendingIndex = controller.GetPendingEntryIndex();
 
     output << kHistoryStreamVersion;
@@ -262,7 +279,7 @@ static void deserializeNavigationHistory(QDataStream &input, int *currentIndex,
     int count;
     input >> count >> *currentIndex;
 
-    std::unique_ptr<content::NavigationEntryRestoreContext> context = content::NavigationEntryRestoreContext::Create(); // FIXME?
+    std::unique_ptr<content::NavigationEntryRestoreContext> context = content::NavigationEntryRestoreContext::Create();
 
     entries->reserve(count);
     // Logic taken from SerializedNavigationEntry::ReadFromPickle and ToNavigationEntries.
@@ -304,6 +321,7 @@ static void deserializeNavigationHistory(QDataStream &input, int *currentIndex,
             toGurl(virtualUrl),
             content::Referrer(toGurl(referrerUrl), static_cast<network::mojom::ReferrerPolicy>(referrerPolicy)),
             absl::nullopt, // optional initiator_origin
+            absl::nullopt, // optional initiator_base_url
             // Use a transition type of reload so that we don't incorrectly
             // increase the typed count.
             ui::PAGE_TRANSITION_RELOAD,
@@ -389,17 +407,6 @@ QSharedPointer<WebContentsAdapter> WebContentsAdapter::createFromSerializedNavig
     content::NavigationController &controller = newWebContents->GetController();
     controller.Restore(currentIndex, content::RestoreType::kRestored, &entries);
 
-    if (controller.GetActiveEntry()) {
-        // Set up the file access rights for the selected navigation entry.
-        // TODO(joth): This is duplicated from chrome/.../session_restore.cc and
-        // should be shared e.g. in  NavigationController. http://crbug.com/68222
-        const int id = newWebContents->GetPrimaryMainFrame()->GetProcess()->GetID();
-        const blink::PageState& pageState = controller.GetActiveEntry()->GetPageState();
-        const std::vector<base::FilePath>& filePaths = pageState.GetReferencedFiles();
-        for (std::vector<base::FilePath>::const_iterator file = filePaths.begin(); file != filePaths.end(); ++file)
-            content::ChildProcessSecurityPolicy::GetInstance()->GrantReadFile(id, *file);
-    }
-
     return QSharedPointer<WebContentsAdapter>::create(std::move(newWebContents));
 }
 
@@ -447,6 +454,10 @@ bool WebContentsAdapter::isInitialized() const
     return (bool)m_webContentsDelegate;
 }
 
+ui::NativeTheme::PreferredColorScheme toWeb(Qt::ColorScheme colorScheme) {
+    return colorScheme == Qt::ColorScheme::Dark ? ui::NativeTheme::PreferredColorScheme::kDark : ui::NativeTheme::PreferredColorScheme::kLight;
+}
+
 void WebContentsAdapter::initialize(content::SiteInstance *site)
 {
     Q_ASSERT(m_adapterClient);
@@ -484,14 +495,6 @@ void WebContentsAdapter::initialize(content::SiteInstance *site)
             webContents(), FaviconServiceFactoryQt::GetForBrowserContext(context), m_adapterClient);
 
     AutofillClientQt::CreateForWebContents(webContents());
-    autofill::ContentAutofillDriverFactory::CreateForWebContentsAndDelegate(
-            webContents(), AutofillClientQt::FromWebContents(webContents()),
-             base::BindRepeating(&autofill::BrowserDriverInitHook, AutofillClientQt::FromWebContents(webContents()), ""));
-
-#if QT_CONFIG(webengine_printing_and_pdf) && QT_CONFIG(webengine_extensions)
-    pdf::PDFWebContentsHelper::CreateForWebContentsWithClient(
-            webContents(), std::make_unique<PDFWebContentsHelperClientQt>());
-#endif
 
     // Create an instance of WebEngineVisitedLinksManager to catch the first
     // content::NOTIFICATION_RENDERER_PROCESS_CREATED event. This event will
@@ -508,6 +511,12 @@ void WebContentsAdapter::initialize(content::SiteInstance *site)
 
     m_webContentsDelegate->RenderViewHostChanged(nullptr, rvh);
 
+    // Make sure the system theme's light/dark mode is propagated to webpages
+    QObject::connect(QGuiApplication::styleHints(), &QStyleHints::colorSchemeChanged, [](Qt::ColorScheme colorScheme){
+        ui::NativeTheme::GetInstanceForWeb()->set_preferred_color_scheme(toWeb(colorScheme));
+    });
+    ui::NativeTheme::GetInstanceForWeb()->set_preferred_color_scheme(toWeb(QGuiApplication::styleHints()->colorScheme()));
+
     m_adapterClient->initializationFinished();
 }
 
@@ -535,6 +544,8 @@ void WebContentsAdapter::initializeRenderPrefs()
                 ? blink::kWebRTCIPHandlingDefaultPublicInterfaceOnly
                 : blink::kWebRTCIPHandlingDefault;
 #endif
+    rendererPrefs->can_accept_load_drops = m_adapterClient->webEngineSettings()->testAttribute(QWebEngineSettings::NavigateOnDropEnabled);
+
     // Set web-contents font settings to the default font settings as Chromium constantly overrides
     // the global font defaults with the font settings of the latest web-contents created.
     static const gfx::FontRenderParams params = gfx::GetFontRenderParams(gfx::FontRenderParamsQuery(), nullptr);
@@ -753,7 +764,7 @@ void WebContentsAdapter::save(const QString &filePath, int savePageFormat)
 {
     CHECK_INITIALIZED();
     base::RecordAction(base::UserMetricsAction("SavePage"));
-    m_webContentsDelegate->setSavePageInfo(SavePageInfo(filePath, savePageFormat));
+    m_webContentsDelegate->setSavePageInfo(new SavePageInfo(filePath, savePageFormat));
     m_webContents->OnSavePage();
 }
 
@@ -909,13 +920,13 @@ void WebContentsAdapter::navigateToOffset(int offset)
 int WebContentsAdapter::navigationEntryCount()
 {
     CHECK_INITIALIZED(0);
-    return m_webContents->GetController().GetEntryCount();
+    return navigationListSize(m_webContents->GetController());
 }
 
 int WebContentsAdapter::currentNavigationEntryIndex()
 {
     CHECK_INITIALIZED(0);
-    return m_webContents->GetController().GetCurrentEntryIndex();
+    return navigationListCurrentIndex(m_webContents->GetController());
 }
 
 QUrl WebContentsAdapter::getNavigationEntryOriginalUrl(int index)
@@ -979,9 +990,8 @@ void WebContentsAdapter::setZoomFactor(qreal factor)
     content::HostZoomMap *zoomMap = content::HostZoomMap::GetForWebContents(m_webContents.get());
 
     if (zoomMap) {
-        int render_process_id = m_webContents->GetPrimaryMainFrame()->GetProcess()->GetID();
-        int render_view_id = m_webContents->GetRenderViewHost()->GetRoutingID();
-        zoomMap->SetTemporaryZoomLevel(render_process_id, render_view_id, zoomLevel);
+        const content::GlobalRenderFrameHostId global_id = m_webContents->GetPrimaryMainFrame()->GetGlobalId();
+        zoomMap->SetTemporaryZoomLevel(global_id, zoomLevel);
     }
 }
 
@@ -1251,7 +1261,10 @@ void WebContentsAdapter::openDevToolsFrontend(QSharedPointer<WebContentsAdapter>
 
     setLifecycleState(LifecycleState::Active);
 
-    m_devToolsFrontend = DevToolsFrontendQt::Show(frontendAdapter, m_webContents.get());
+    content::WebContents *webContents = m_webContents.get();
+    if (content::WebContents *guest = guestWebContents())
+        webContents = guest;
+    m_devToolsFrontend = DevToolsFrontendQt::Show(frontendAdapter, webContents);
     updateRecommendedState();
 }
 
@@ -1271,6 +1284,11 @@ void WebContentsAdapter::devToolsFrontendDestroyed(DevToolsFrontendQt *frontend)
     updateRecommendedState();
 }
 
+QString WebContentsAdapter::devToolsId()
+{
+    return QString::fromStdString(DevToolsFrontendQt::GetId(m_webContents.get()));
+}
+
 void WebContentsAdapter::exitFullScreen()
 {
     CHECK_INITIALIZED();
@@ -1531,7 +1549,7 @@ void WebContentsAdapter::startDragging(QObject *dragSource, const content::DropD
     }
 
     {
-        base::CurrentThread::ScopedNestableTaskAllower allow;
+        base::CurrentThread::ScopedAllowApplicationTasksInNativeNestedLoop allow;
         drag->exec(allowedActions);
     }
 
@@ -1601,7 +1619,7 @@ static void fillDropDataFromMimeData(content::DropData *dropData, const QMimeDat
     }
     if (!dropData->filenames.empty())
         return;
-    if (mimeData->hasUrls()) {
+    if (!urls.empty()) {
         dropData->url = toGurl(urls.first());
         if (mimeData->hasText())
             dropData->url_title = toString16(mimeData->text());
@@ -1780,6 +1798,20 @@ void WebContentsAdapter::resetTouchSelectionController()
         rwhv->resetTouchSelectionController();
 }
 
+void WebContentsAdapter::changeTextDirection(bool leftToRight)
+{
+    CHECK_INITIALIZED();
+    if (auto rwhv = static_cast<RenderWidgetHostViewQt *>(m_webContents->GetRenderWidgetHostView())) {
+        auto textInputManager = rwhv->GetTextInputManager();
+        if (!textInputManager)
+            return;
+        if (auto activeWidget = textInputManager->GetActiveWidget()) {
+            activeWidget->UpdateTextDirection(leftToRight ? base::i18n::TextDirection::LEFT_TO_RIGHT : base::i18n::TextDirection::RIGHT_TO_LEFT);
+            activeWidget->NotifyTextDirection();
+        }
+    }
+}
+
 WebContentsAdapterClient::RenderProcessTerminationStatus
 WebContentsAdapterClient::renderProcessExitStatus(int terminationStatus) {
     auto status = WebContentsAdapterClient::RenderProcessTerminationStatus(-1);
@@ -1935,7 +1967,7 @@ WebContentsAdapter::LifecycleState WebContentsAdapter::determineRecommendedState
     // Do not discard PDFs as they might contain entry that is not saved and they
     // don't remember their scrolling positions. See crbug.com/547286 and
     // crbug.com/65244.
-    if (m_webContents->GetContentsMimeType() == "application/pdf")
+    if (m_webContents->GetContentsMimeType() == kPDFMimeType)
         return LifecycleState::Frozen;
 
     return LifecycleState::Discarded;
