@@ -432,6 +432,35 @@ bool QQuickWebEngineSettings::navigateOnDropEnabled() const
     return d_ptr->testAttribute(QWebEngineSettings::NavigateOnDropEnabled);
 }
 
+/*!
+    \qmlproperty bool WebEngineSettings::readingFromCanvasEnabled
+    \since QtWebEngine 6.6
+
+    Specifies that reading from all canvas elements is enabled.
+
+    This setting will have impact on all HTML5 canvas elements irrespective of origin, and can be disabled
+    to prevent canvas fingerprinting.
+
+    Enabled by default.
+ */
+bool QQuickWebEngineSettings::readingFromCanvasEnabled() const
+{
+    return d_ptr->testAttribute(QWebEngineSettings::ReadingFromCanvasEnabled);
+}
+
+/*!
+    \qmlproperty bool WebEngineSettings::forceDarkMode
+    \since QtWebEngine 6.7
+
+    Automatically render all web contents using a dark theme.
+
+    Disabled by default.
+ */
+bool QQuickWebEngineSettings::forceDarkMode() const
+{
+    return d_ptr->testAttribute(QWebEngineSettings::ForceDarkMode);
+}
+
 /*!
     \qmlproperty string WebEngineSettings::defaultTextEncoding
     \since QtWebEngine 1.2
@@ -715,6 +744,22 @@ void QQuickWebEngineSettings::setNavigateOnDropEnabled(bool on)
         Q_EMIT navigateOnDropEnabledChanged();
 }
 
+void QQuickWebEngineSettings::setReadingFromCanvasEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(QWebEngineSettings::ReadingFromCanvasEnabled);
+    d_ptr->setAttribute(QWebEngineSettings::ReadingFromCanvasEnabled, on);
+    if (wasOn != on)
+        Q_EMIT readingFromCanvasEnabledChanged();
+}
+
+void QQuickWebEngineSettings::setForceDarkMode(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(QWebEngineSettings::ForceDarkMode);
+    d_ptr->setAttribute(QWebEngineSettings::ForceDarkMode, on);
+    if (wasOn != on)
+        Q_EMIT forceDarkModeChanged();
+}
+
 void QQuickWebEngineSettings::setUnknownUrlSchemePolicy(QQuickWebEngineSettings::UnknownUrlSchemePolicy policy)
 {
     QWebEngineSettings::UnknownUrlSchemePolicy oldPolicy = d_ptr->unknownUrlSchemePolicy();
